# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

import copy
import os
import shutil
import signal
import tempfile
import time
from contextlib import contextmanager
from io import StringIO
from pathlib import Path
from typing import Callable, Dict, List, Optional, Union
from unittest.mock import patch
from zipfile import ZipFile

import pydash
import urllib3
from azure.core.exceptions import HttpResponseError
from azure.core.polling import LROPoller
from devtools_testutils import is_live

from azure.ai.ml import MLClient, load_job
from azure.ai.ml._scope_dependent_operations import OperationScope
from azure.ai.ml.entities import Job, PipelineJob
from azure.ai.ml.operations._job_ops_helper import _wait_before_polling
from azure.ai.ml.operations._run_history_constants import JobStatus, RunHistoryConstants

_PYTEST_TIMEOUT_METHOD = "signal" if hasattr(signal, "SIGALRM") else "thread"  # use signal when os support SIGALRM
DEFAULT_TASK_TIMEOUT = 30 * 60  # 30mins
THREAD_WAIT_TIME_BEFORE_POLL = 60  # 1min


def write_script(script_path: str, content: str) -> str:
    """
    Util for generating a python script, currently writes the file to disk.
    """
    with open(script_path, "w") as stream:
        stream.write(content)
    return script_path


def get_arm_id(ws_scope: OperationScope, entity_name: str, entity_version: str, entity_type) -> str:
    arm_id = (
        "/subscriptions/{}/resourceGroups/{}/providers/Microsoft.MachineLearningServices/workspaces/{"
        "}/{}/{}/versions/{}".format(
            ws_scope.subscription_id,
            ws_scope.resource_group_name,
            ws_scope.workspace_name,
            entity_type,
            entity_name,
            entity_version,
        )
    )

    return arm_id


def omit_single_with_wildcard(obj, omit_field: str):
    """
    Support .*. for pydash.omit
        omit_with_wildcard({"a": {"1": {"b": "v"}, "2": {"b": "v"}}}, "a.*.b")
        {"a": {"1": {}, "2": {}}}
    """
    obj = copy.deepcopy(obj)
    target_mark = ".*."
    if target_mark in omit_field:
        prefix, next_omit_field = omit_field.split(target_mark, 1)
        new_obj = pydash.get(obj, prefix)
        if new_obj:
            for key, value in new_obj.items():
                new_obj[key] = omit_single_with_wildcard(value, next_omit_field)
            pydash.set_(obj, prefix, new_obj)
        return obj
    return pydash.omit(obj, omit_field)


def omit_with_wildcard(obj, *properties: str):
    for omit_field in properties:
        obj = omit_single_with_wildcard(obj, omit_field)
    return obj


def prepare_dsl_curated(
    pipeline: PipelineJob, job_yaml, omit_fields=None, enable_default_omit_fields=True, in_rest=False
):
    """
    Prepare the dsl pipeline for curated test.
    Return objs instead of assert equal directly to enable difference viewer in PyCharm.
    """
    if omit_fields is None:
        omit_fields = []
    pipeline_from_yaml = load_job(source=job_yaml)
    if in_rest:
        dsl_pipeline_job_dict = pipeline._to_rest_object().as_dict()  # pylint: disable=protected-access
        pipeline_job_dict = pipeline_from_yaml._to_rest_object().as_dict()  # pylint: disable=protected-access

        if enable_default_omit_fields:
            omit_fields.extend(
                [
                    "name",
                    "properties.jobs.*.componentId",
                    "properties.jobs.*._source",
                    "properties.jobs.*.trial.properties.componentSpec.name",
                    "properties.jobs.*.trial.properties.componentSpec.version",
                    "properties.jobs.*.trial.properties.componentSpec.$schema",
                    "properties.jobs.*.trial.properties.componentSpec.schema",
                    "properties.jobs.*.trial.properties.isAnonymous",
                    "properties.jobs.*.trial.properties.componentSpec._source",
                    "properties.settings",
                    "properties.jobs.*.trial.properties.properties.client_component_hash",
                ]
            )
    else:
        dsl_pipeline_job_dict = pipeline._to_dict()  # pylint: disable=protected-access
        pipeline_job_dict = pipeline_from_yaml._to_dict()  # pylint: disable=protected-access
        if enable_default_omit_fields:
            omit_fields.extend(
                [
                    "name",
                    "jobs.*.component.name",
                    "jobs.*.component.version",
                    "jobs.*.trial.name",
                    "jobs.*.trial.version",
                ]
            )

    dsl_pipeline_job_dict = omit_with_wildcard(dsl_pipeline_job_dict, *omit_fields)
    pipeline_job_dict = omit_with_wildcard(pipeline_job_dict, *omit_fields)

    return dsl_pipeline_job_dict, pipeline_job_dict


def submit_and_wait(ml_client, pipeline_job: PipelineJob, expected_state: str = "Completed") -> PipelineJob:
    created_job = ml_client.jobs.create_or_update(pipeline_job)
    terminal_states = ["Completed", "Failed", "Canceled", "NotResponding"]
    assert created_job is not None
    assert expected_state in terminal_states

    while created_job.status not in terminal_states:
        sleep_if_live(30)
        created_job = ml_client.jobs.get(created_job.name)
        print("Latest status : {0}".format(created_job.status))
    if created_job.status != expected_state:
        raise Exception(
            f"Job finished with unexpected status. Got {created_job.status!r} while expecting {expected_state!r}"
        )
    print(f"Job finished: {expected_state!r}")
    assert created_job.status == expected_state
    return created_job


def assert_final_job_status(
    job, client: MLClient, job_type: Job, expected_terminal_status: str, deadline: int = DEFAULT_TASK_TIMEOUT
) -> None:
    assert isinstance(job, job_type)

    poll_start_time = time.time()
    while job.status not in RunHistoryConstants.TERMINAL_STATUSES and time.time() < (poll_start_time + deadline):
        sleep_if_live(THREAD_WAIT_TIME_BEFORE_POLL)
        job = client.jobs.get(job.name)

    if job.status not in RunHistoryConstants.TERMINAL_STATUSES:
        cancel_poller = client.jobs.begin_cancel(job.name)
        assert isinstance(cancel_poller, LROPoller)
        assert cancel_poller.result() is None

    assert job.status == expected_terminal_status, f"Job status mismatch. Job created: {job}"


def get_automl_job_properties() -> Dict:
    return {}


def download_dataset(download_url: str, data_file: str, retries=3) -> None:
    # download data
    http = urllib3.PoolManager()
    resp = http.request("GET", download_url, preload_content=False, retries=retries)
    with open(data_file, "wb") as f:
        for chunk in resp.stream(1024):
            f.write(chunk)
    resp.release_conn()

    # extract files
    with ZipFile(data_file, "r") as _zip:
        print("extracting files...")
        _zip.extractall()
        print("done")
    # delete zip file
    os.remove(data_file)


# Given an entity's load function, load it using an open file input, a file path,
# and even a deprecated 'path=' path input then run the inputted
# entity validation function on all resulting entities.
# After validation, dump the loaded entity using all three input types
# and assert that the resulting dumped file is non-empty.
# Returns both path and pointer-loaded entities to allow for additional testing
# by the caller.
def verify_entity_load_and_dump(
    load_function,
    entity_validation_function,
    test_load_file_path: str,
    test_dump_file_path="dump-test.yaml",
    **load_kwargs,
):
    # test loading
    file_entity, stream_entity = None, None
    with open(test_load_file_path, "r") as f:
        file_entity = load_function(source=f, **load_kwargs)
    file_contents = ""
    with open(test_load_file_path, "r") as f:
        file_contents = f.read()
    with StringIO() as stream:
        stream.write(file_contents)
        stream.seek(0)
        stream_entity = load_function(source=stream, relative_origin=test_load_file_path, **load_kwargs)

    assert file_entity is not None
    assert stream_entity is not None

    first_input_entity = load_function(test_load_file_path, **load_kwargs)
    assert first_input_entity is not None

    old_path_entity = load_function(test_load_file_path, **load_kwargs)
    assert old_path_entity is not None

    # Run entity-specific validation on both loaded entities
    entity_validation_function(file_entity)
    entity_validation_function(stream_entity)
    entity_validation_function(first_input_entity)
    entity_validation_function(old_path_entity)

    # test dump
    # TODO once dump functionality audit is complete, this testing should be
    # made more robust, like comparing it to the inputted yaml or something.
    if test_dump_file_path is not None:
        # test file pointer-based dump
        with tempfile.TemporaryDirectory() as tmpdirname:
            tmpfilename = f"{tmpdirname}/{test_dump_file_path}"
            with open(tmpfilename, "w+") as f:
                file_entity.dump(f)
            assert get_file_contents(tmpfilename) != ""

        # test string stream dump
        with StringIO() as outputStream:
            stream_entity.dump(outputStream)
            assert outputStream.tell() > 0

        # test path-based dump
        with tempfile.TemporaryDirectory() as tmpdirname:
            tmpfilename = f"{tmpdirname}/{test_dump_file_path}"
            first_input_entity.dump(tmpfilename)
            assert get_file_contents(tmpfilename) != ""
            delete_file_if_exists(tmpfilename)

        # test path-based dump using deprecated input name
        with tempfile.TemporaryDirectory() as tmpdirname:
            tmpfilename = f"{tmpdirname}/{test_dump_file_path}"
            first_input_entity.dump(tmpfilename)
            assert get_file_contents(tmpfilename) != ""
            delete_file_if_exists(tmpfilename)

    return (old_path_entity, file_entity)


def get_file_contents(file_path: str):
    if os.path.exists(file_path):
        with open(file_path, "r") as f:
            return f.read()
    return ""


def delete_file_if_exists(file_path: str):
    if file_path is not None and os.path.exists(file_path):
        os.remove(file_path)


def cancel_job(client: MLClient, job: Job) -> None:
    try:
        cancel_poller = client.jobs.begin_cancel(job.name)
        assert isinstance(cancel_poller, LROPoller)
        assert cancel_poller.result() is None
    except HttpResponseError:
        pass


def assert_job_cancel(
    job: Job,
    client: MLClient,
    *,
    experiment_name=None,
    check_before_cancelled: Callable[[Job], bool] = None,
    skip_cancel=False,
    wait_for_completion=False,
) -> Job:
    created_job = client.jobs.create_or_update(job, experiment_name=experiment_name)
    if check_before_cancelled is not None:
        assert check_before_cancelled(created_job)
    if skip_cancel is False:
        cancel_job(client, created_job)
    elif wait_for_completion is True:
        assert wait_until_done(client, created_job) == JobStatus.COMPLETED, (
            "Job failed. Please check it on studio for more details: %s" % created_job.studio_url
        )
    return created_job


def submit_and_cancel_new_dsl_pipeline(pipeline_func, client, default_compute="cpu-cluster", **kwargs):
    pipeline_job: PipelineJob = pipeline_func(**kwargs)
    pipeline_job.settings.default_compute = default_compute
    return assert_job_cancel(pipeline_job, client)


def wait_until_done(client: MLClient, job: Job, timeout: int = None) -> str:
    poll_start_time = time.time()
    while job.status not in RunHistoryConstants.TERMINAL_STATUSES:
        sleep_if_live(_wait_before_polling(time.time() - poll_start_time))
        job = client.jobs.get(job.name)
        if timeout is not None and time.time() - poll_start_time > timeout:
            # if timeout is passed in, execute job cancel if timeout and directly return CANCELED status
            cancel_job(client, job)
            return JobStatus.CANCELED
    return job.status


def sleep_if_live(seconds):
    """Sleeps for the given number of seconds if the test is live.
    In playback mode, this function does nothing.
    Please use this function instead of time.sleep() in tests if you want to wait for some remote operations.

    Not necessary actually when fixture skip_sleep_for_playback has not been disabled explicitly.
    Unify the behavior in case the fixture is disabled, like switch to skip_sleep_in_lro_polling.
    """
    if is_live():
        time.sleep(seconds)


def parse_local_path(origin_path, base_path=None):
    """Relative path in LocalPathField will be dumped to absolute path in _to_dict.
    In e2e tests, it will be uploaded and resolved into an uri before _to_rest_object.
    However, developers usually directly compare the result of _to_dict/_to_rest_object with a fixed dict.
    So we provide this function to parse the original value in the same way as LocalPathField serialization
    to avoid updating test cases after updating LocalPathField serialization logic.

    :param origin_path: The original value filled in LocalPathField
    :param base_path: The base path of the original value, usually from resource.base_path.
    Will use current working directory if not provided.
    """
    if base_path is None:
        base_path = Path.cwd()
    else:
        base_path = Path(base_path)
    return (base_path / origin_path).resolve().absolute().as_posix()


@contextmanager
def build_temp_folder(
    *,
    source_base_dir: Union[str, os.PathLike] = None,
    relative_dirs_to_copy: List[str] = None,
    relative_files_to_copy: List[str] = None,
    extra_files_to_create: Dict[str, Union[None, bytes, str]] = None,
) -> str:
    """Build a temporary folder with files and subfolders copied from source_base_dir.
    Note that the last part of path to upload will be recorded as part of request url in playback mode, so please avoid
    directly use returned temp folder as component.code.

    :param source_base_dir: The base directory to copy files from.
    :type source_base_dir: Union[str, os.PathLike]
    :param relative_dirs_to_copy: The relative paths of subfolders to copy from source_base_dir.
    :type relative_dirs_to_copy: List[str]
    :param relative_files_to_copy: The relative paths of files to copy from source_base_dir.
    :type relative_files_to_copy: List[str]
    :param extra_files_to_create: The relative paths of files to create in the temporary folder.
    The value of each key-value pair is the content of the file.
    If the value is None, the file will be empty.
    :type extra_files_to_create: Dict[str, Optional[str]]
    :return: The path of the temporary folder.
    :rtype: str
    """
    if source_base_dir is None:
        source_base_dir = Path.cwd()
    else:
        source_base_dir = Path(source_base_dir)
    with tempfile.TemporaryDirectory() as temp_dir:
        if relative_dirs_to_copy:
            for dir_name in relative_dirs_to_copy:
                shutil.copytree(source_base_dir / dir_name, Path(temp_dir) / dir_name)
        if relative_files_to_copy:
            for file_name in relative_files_to_copy:
                shutil.copy(source_base_dir / file_name, Path(temp_dir) / file_name)
        if extra_files_to_create:
            for file_name, content in extra_files_to_create.items():
                target_file = Path(temp_dir) / file_name
                target_file.parent.mkdir(parents=True, exist_ok=True)
                if content is None:
                    target_file.touch()
                    continue
                if content is None:
                    target_file.touch()
                elif isinstance(content, str):
                    target_file.write_text(content)
                elif isinstance(content, bytes):
                    target_file.write_bytes(content)
                else:
                    raise ValueError(f"Unsupported content type {type(content)}")

        yield temp_dir


@contextmanager
def reload_schema_for_nodes_in_pipeline_job(*, revert_after_yield: bool = True):
    """Reload schema for nodes in pipeline job. This is needed when we want to test private preview features or
    unregister internal components.

    This method should be called after environment variable is set, so we make it a method instead of a fixture.
    """
    # Update the node types in pipeline jobs to include the private preview node types
    from azure.ai.ml._schema.pipeline import pipeline_job

    declared_fields = pipeline_job.PipelineJobSchema._declared_fields  # pylint: disable=protected-access, no-member
    original_jobs = declared_fields["jobs"]
    declared_fields["jobs"] = pipeline_job.PipelineJobsField()

    try:
        yield
    finally:
        if revert_after_yield:
            declared_fields["jobs"] = original_jobs


@contextmanager
def mock_artifact_download_to_temp_directory():
    with tempfile.TemporaryDirectory() as temp_dir:

        def mock_get_artifacts(**kwargs):
            version = kwargs.get("version")
            artifact = Path(temp_dir) / version
            if version in ["version_1", "version_3"]:
                version = "version_1"
            artifact.mkdir(parents=True, exist_ok=True)
            (artifact / version).mkdir(exist_ok=True)
            (artifact / version / "file").touch(exist_ok=True)
            (artifact / f"file_{version}").touch(exist_ok=True)
            return str(artifact)

        with patch("azure.ai.ml._utils._artifact_utils.ArtifactCache.get", side_effect=mock_get_artifacts):
            yield temp_dir



import datetime

import pytest

from azure.ai.ml._restclient.azure_ai_assets_v2024_04_01.azureaiassetsv20240401.models import Index as RestIndex
from azure.ai.ml._restclient.azure_ai_assets_v2024_04_01.azureaiassetsv20240401.models import (
    SystemData as RestSystemData,
)
from azure.ai.ml.entities import Index


@pytest.mark.unittest
class TestIndexEntity:
    def test_index_from_rest(self) -> None:
        rest_index = RestIndex(
            stage="Development",
            description="Hello World",
            tags={"tag1": "foo", "tag2": "bar"},
            properties={"prop1": "foo", "prop2": "bar"},
            id="azureml://locations/eastus/workspaces/my_workspace/indexes/my_index/versions/1",
            storage_uri=(
                "https://mystorageaccount.blob.core.windows.net/"
                + "XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX-azureml-blobstore/"
                + "LocalUpload/XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/index"
            ),
            system_data=RestSystemData(
                created_at=datetime.datetime(2024, 1, 1),
                created_by="Firstname Lastname",
                created_by_type="User",
                last_modified_at=datetime.datetime(2024, 1, 1),
            ),
        )

        index = Index._from_rest_object(rest_index)

        assert index.stage == "Development"
        assert index.description == "Hello World"

        # FIXME: Every other asset in the SDK exposes the ARM ID and not the asset ID
        assert index.id == "azureml://locations/eastus/workspaces/my_workspace/indexes/my_index/versions/1"

        assert index.name == "my_index"
        assert index.version == "1"
        assert index.stage == "Development"
        assert index.tags == {"tag1": "foo", "tag2": "bar"}
        assert index.properties == {"prop1": "foo", "prop2": "bar"}
        assert index.path == (
            "https://mystorageaccount.blob.core.windows.net/XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX-azureml-blobstore/"
            + "LocalUpload/XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/index"
        )

    def test_index_to_rest(self) -> None:
        index = Index(
            name="foo",
            version="1",
            stage="Development",
            description="Hello World",
            tags={"tag1": "foo", "tag2": "bar"},
            properties={"prop1": "foo", "prop2": "bar"},
            path=(
                "https://mystorageaccount.blob.core.windows.net/"
                + "XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX-azureml-blobstore/"
                + "LocalUpload/XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/index"
            ),
        )

        rest_index = index._to_rest_object()

        assert rest_index.stage == "Development"
        assert rest_index.description == "Hello World"
        assert rest_index.stage == "Development"
        assert rest_index.tags == {"tag1": "foo", "tag2": "bar"}
        assert rest_index.properties == {"prop1": "foo", "prop2": "bar"}
        assert rest_index.system_data is None
        assert rest_index.id is None
        assert rest_index.storage_uri == (
            "https://mystorageaccount.blob.core.windows.net/"
            + "XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX-azureml-blobstore/"
            + "LocalUpload/XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/index"
        )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from typing import Any, Dict, Optional

from azure.ai.ml._schema._deployment.online.payload_response_schema import PayloadResponseSchema
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY


class PayloadResponse:
    """Response deployment entity

    :param enabled: Is response logging enabled.
    :type enabled: str

    """

    # pylint: disable=unused-argument
    def __init__(self, enabled: Optional[str] = None, **kwargs: Any):
        self.enabled = enabled

    def _to_dict(self) -> Dict:
        # pylint: disable=no-member
        res: dict = PayloadResponseSchema(context={BASE_PATH_CONTEXT_KEY: "./"}).dump(self)
        return res



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

"""This file stores functions and objects that will be used in mldesigner package.

DO NOT change the module names in "all" list. If the interface has changed in source code, wrap it here and keep
original function/module names the same as before, otherwise mldesigner will be broken by this change.
"""

__path__ = __import__("pkgutil").extend_path(__path__, __name__)

from azure.ai.ml._internal.entities import InternalComponent
from azure.ai.ml._internal.entities._additional_includes import InternalAdditionalIncludes
from azure.ai.ml._utils._asset_utils import get_ignore_file
from azure.ai.ml._utils.utils import try_enable_internal_components
from azure.ai.ml.dsl._condition import condition
from azure.ai.ml.dsl._do_while import do_while
from azure.ai.ml.dsl._group_decorator import group
from azure.ai.ml.dsl._parallel_for import ParallelFor, parallel_for
from azure.ai.ml.entities._component.component_factory import component_factory
from azure.ai.ml.entities._inputs_outputs import _get_param_with_standard_annotation
from azure.ai.ml.entities._job.pipeline._load_component import _generate_component_function

from ._constants import V1_COMPONENT_TO_NODE

component_factory_load_from_dict = component_factory.load_from_dict


__all__ = [
    # to be put in main package
    "condition",
    "do_while",
    "group",
    "parallel_for",
    "ParallelFor",
    # must keep
    "get_ignore_file",
    "_get_param_with_standard_annotation",
    "_generate_component_function",
    "component_factory_load_from_dict",
    "V1_COMPONENT_TO_NODE",
    "try_enable_internal_components",
    "InternalAdditionalIncludes",
    "InternalComponent",
]



from unittest.mock import Mock, patch

import pytest
from pytest_mock import MockFixture

from azure.ai.ml._scope_dependent_operations import OperationConfig, OperationsContainer, OperationScope
from azure.ai.ml.constants._common import AzureMLResourceType
from azure.ai.ml.entities._assets import Code
from azure.ai.ml.exceptions import ValidationException
from azure.ai.ml.operations import ComponentOperations
from azure.ai.ml.operations._code_operations import CodeOperations
from azure.ai.ml.operations._operation_orchestrator import OperationOrchestrator
from azure.core.exceptions import HttpResponseError


class MockResponse:
    def __init__(self, code, reason=None):
        self.status_code = code
        self.reason = reason

    def raise_for_status(self):
        return


@pytest.fixture
def mock_component_operation(
    mock_workspace_scope: OperationScope,
    mock_operation_config: OperationConfig,
    mock_aml_services_2022_05_01: Mock,
    mock_machinelearning_client: Mock,
) -> ComponentOperations:
    yield ComponentOperations(
        operation_scope=mock_workspace_scope,
        operation_config=mock_operation_config,
        service_client=mock_aml_services_2022_05_01,
        all_operations=mock_machinelearning_client._operation_container,
    )


@pytest.fixture
def code_operations(mocker: MockFixture) -> Mock:
    return mocker.patch("azure.ai.ml.operations._code_operations.CodeOperations")


@pytest.fixture
def operation_container(
    code_operations: CodeOperations,
) -> OperationsContainer:
    container = OperationsContainer()
    container.add(AzureMLResourceType.CODE, code_operations)
    yield container


@pytest.fixture
def operation_orchestrator(
    mock_workspace_scope: OperationScope,
    mock_operation_config: OperationConfig,
    operation_container: OperationsContainer,
) -> OperationOrchestrator:
    yield OperationOrchestrator(
        operation_container=operation_container,
        operation_scope=mock_workspace_scope,
        operation_config=mock_operation_config,
    )


@pytest.mark.unittest
@pytest.mark.core_sdk_test
class TestExceptions:
    def test_error_preprocess(self, mock_component_operation: ComponentOperations):
        with patch.object(mock_component_operation._version_operation, "get") as mock_client:
            mock_client.side_effect = HttpResponseError(response=MockResponse(code=400))
            with pytest.raises(HttpResponseError) as context:
                mock_component_operation.get("mock_component", "0.0.1")
            assert context.type.__name__ == "HttpResponseError"

            mock_client.side_effect = KeyboardInterrupt()
            with pytest.raises(BaseException) as context:
                mock_component_operation.get("mock_component", "0.0.1")
            assert context.type.__name__ == "KeyboardInterrupt"

    def test_get_code_asset_arm_id(self, operation_orchestrator: OperationOrchestrator):
        with patch.object(operation_orchestrator, "_validate_datastore_name") as mock_client:
            mock_client.side_effect = ValidationException(
                message="raise ValidationException error", no_personal_data_message="raise ValidationException error"
            )
            with pytest.raises(ValidationException) as context:
                operation_orchestrator.get_asset_arm_id(asset=Code(), azureml_type=AzureMLResourceType.CODE)
            assert context.type.__name__ == "ValidationException"



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------


import logging
from typing import Any

from marshmallow import fields, post_load

from azure.ai.ml._schema import (
    ArmVersionedStr,
    PatchedSchemaMeta,
    StringTransformedEnum,
    UnionField,
    ArmStr,
    RegistryStr,
)
from azure.ai.ml._schema.pipeline.pipeline_component import PipelineComponentFileRefField
from azure.ai.ml.constants._common import AzureMLResourceType
from azure.ai.ml.constants._job.job import JobType

module_logger = logging.getLogger(__name__)


# pylint: disable-next=name-too-long
class BatchPipelineComponentDeploymentConfiguarationsSchema(metaclass=PatchedSchemaMeta):
    component_id = fields.Str()
    job = UnionField(
        [
            ArmStr(azureml_type=AzureMLResourceType.JOB),
            PipelineComponentFileRefField(),
        ]
    )
    component = UnionField(
        [
            RegistryStr(azureml_type=AzureMLResourceType.COMPONENT),
            ArmVersionedStr(azureml_type=AzureMLResourceType.COMPONENT, allow_default_version=True),
            PipelineComponentFileRefField(),
        ]
    )
    type = StringTransformedEnum(required=True, allowed_values=[JobType.PIPELINE])
    settings = fields.Dict()
    name = fields.Str()
    description = fields.Str()
    tags = fields.Dict()

    @post_load
    def make(self, data: Any, **kwargs: Any) -> Any:  # pylint: disable=unused-argument
        from azure.ai.ml.entities._deployment.job_definition import JobDefinition

        return JobDefinition(**data)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument

from marshmallow import ValidationError, fields, post_load, pre_dump

from azure.ai.ml._schema._sweep._constants import BASE_ERROR_MESSAGE
from azure.ai.ml._schema.core.fields import DumpableIntegerField, StringTransformedEnum, UnionField
from azure.ai.ml._schema.core.schema import PatchedSchemaMeta
from azure.ai.ml.constants._common import TYPE
from azure.ai.ml.constants._job.sweep import SearchSpace


class UniformSchema(metaclass=PatchedSchemaMeta):
    type = StringTransformedEnum(required=True, allowed_values=SearchSpace.UNIFORM_LOGUNIFORM)
    min_value = UnionField([DumpableIntegerField(strict=True), fields.Float()], required=True)
    max_value = UnionField([DumpableIntegerField(strict=True), fields.Float()], required=True)

    @pre_dump
    def predump(self, data, **kwargs):
        from azure.ai.ml.sweep import LogUniform, Uniform

        if not isinstance(data, (Uniform, LogUniform)):
            raise ValidationError("Cannot dump non-Uniform or non-LogUniform object into UniformSchema")
        if data.type.lower() not in SearchSpace.UNIFORM_LOGUNIFORM:
            raise ValidationError(BASE_ERROR_MESSAGE + str(SearchSpace.UNIFORM_LOGUNIFORM))
        return data

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.sweep import LogUniform, Uniform

        return Uniform(**data) if data[TYPE] == SearchSpace.UNIFORM else LogUniform(**data)


class QUniformSchema(metaclass=PatchedSchemaMeta):
    type = StringTransformedEnum(required=True, allowed_values=SearchSpace.QUNIFORM_QLOGUNIFORM)
    min_value = UnionField([DumpableIntegerField(strict=True), fields.Float()], required=True)
    max_value = UnionField([DumpableIntegerField(strict=True), fields.Float()], required=True)
    q = UnionField([DumpableIntegerField(strict=True), fields.Float()], required=True)

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.sweep import QLogUniform, QUniform

        return QUniform(**data) if data[TYPE] == SearchSpace.QUNIFORM else QLogUniform(**data)

    @pre_dump
    def predump(self, data, **kwargs):
        from azure.ai.ml.sweep import QLogUniform, QUniform

        if not isinstance(data, (QUniform, QLogUniform)):
            raise ValidationError("Cannot dump non-QUniform or non-QLogUniform object into UniformSchema")
        return data


class IntegerQUniformSchema(QUniformSchema):
    min_value = DumpableIntegerField(strict=True, required=True)
    max_value = DumpableIntegerField(strict=True, required=True)
    q = DumpableIntegerField(strict=True, required=True)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from os import PathLike
from typing import Any, Optional, Union

from azure.ai.ml._restclient.v2023_10_01.models import FeaturesetSpecification as RestFeaturesetSpecification
from azure.ai.ml.entities._mixins import RestTranslatableMixin


class FeatureSetSpecification(RestTranslatableMixin):
    """Feature Set Specification

    :param path: Specifies the feature set spec path to file. Defaults to None.
    :type path: Optional[str]
    :param kwargs: A dictionary of additional configuration parameters.
    :type kwargs: dict

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_featurestore.py
            :start-after: [START configure_feature_set]
            :end-before: [END configure_feature_set]
            :language: python
            :dedent: 8
            :caption: Using Feature Set Spec to create Feature Set
    """

    def __init__(
        self, *, path: Optional[Union[PathLike, str]] = None, **kwargs: Any
    ):  # pylint: disable=unused-argument
        """
        :param path: Specifies the spec path.
        :type path: str
        """
        self.path = path

    def _to_rest_object(self) -> RestFeaturesetSpecification:
        return RestFeaturesetSpecification(path=self.path)

    @classmethod
    def _from_rest_object(cls, obj: RestFeaturesetSpecification) -> Optional["FeatureSetSpecification"]:
        if not obj:
            return None
        return FeatureSetSpecification(path=obj.path)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
# pylint: disable=unused-import
__path__ = __import__("pkgutil").extend_path(__path__, __name__)

from .component_job import (
    CommandSchema,
    ImportSchema,
    ParallelSchema,
    SparkSchema,
    DataTransferCopySchema,
    DataTransferImportSchema,
    DataTransferExportSchema,
)
from .pipeline_job import PipelineJobSchema
from .settings import PipelineJobSettingsSchema



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import msrest.serialization


class Artifact(msrest.serialization.Model):
    """Artifact.

    :ivar id:
    :vartype id: str
    :ivar prefix:
    :vartype prefix: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'prefix': {'key': 'prefix', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword id:
        :paramtype id: str
        :keyword prefix:
        :paramtype prefix: str
        """
        super(Artifact, self).__init__(**kwargs)
        self.id = kwargs.get('id', None)
        self.prefix = kwargs.get('prefix', None)


class Asset(msrest.serialization.Model):
    """Asset.

    All required parameters must be populated in order to send to Azure.

    :ivar id:
    :vartype id: str
    :ivar name: Required.
    :vartype name: str
    :ivar type:
    :vartype type: str
    :ivar description:
    :vartype description: str
    :ivar artifacts:
    :vartype artifacts: list[~azure.mgmt.machinelearningservices.models.Artifact]
    :ivar kv_tags: Dictionary of :code:`<string>`.
    :vartype kv_tags: dict[str, str]
    :ivar properties: Dictionary of :code:`<string>`.
    :vartype properties: dict[str, str]
    :ivar runid:
    :vartype runid: str
    :ivar projectid:
    :vartype projectid: str
    :ivar meta: Dictionary of :code:`<string>`.
    :vartype meta: dict[str, str]
    :ivar created_time:
    :vartype created_time: ~datetime.datetime
    """

    _validation = {
        'name': {'required': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'artifacts': {'key': 'artifacts', 'type': '[Artifact]'},
        'kv_tags': {'key': 'kvTags', 'type': '{str}'},
        'properties': {'key': 'properties', 'type': '{str}'},
        'runid': {'key': 'runid', 'type': 'str'},
        'projectid': {'key': 'projectid', 'type': 'str'},
        'meta': {'key': 'meta', 'type': '{str}'},
        'created_time': {'key': 'createdTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword id:
        :paramtype id: str
        :keyword name: Required.
        :paramtype name: str
        :keyword type:
        :paramtype type: str
        :keyword description:
        :paramtype description: str
        :keyword artifacts:
        :paramtype artifacts: list[~azure.mgmt.machinelearningservices.models.Artifact]
        :keyword kv_tags: Dictionary of :code:`<string>`.
        :paramtype kv_tags: dict[str, str]
        :keyword properties: Dictionary of :code:`<string>`.
        :paramtype properties: dict[str, str]
        :keyword runid:
        :paramtype runid: str
        :keyword projectid:
        :paramtype projectid: str
        :keyword meta: Dictionary of :code:`<string>`.
        :paramtype meta: dict[str, str]
        :keyword created_time:
        :paramtype created_time: ~datetime.datetime
        """
        super(Asset, self).__init__(**kwargs)
        self.id = kwargs.get('id', None)
        self.name = kwargs['name']
        self.type = kwargs.get('type', None)
        self.description = kwargs.get('description', None)
        self.artifacts = kwargs.get('artifacts', None)
        self.kv_tags = kwargs.get('kv_tags', None)
        self.properties = kwargs.get('properties', None)
        self.runid = kwargs.get('runid', None)
        self.projectid = kwargs.get('projectid', None)
        self.meta = kwargs.get('meta', None)
        self.created_time = kwargs.get('created_time', None)


class AssetDto(msrest.serialization.Model):
    """AssetDto.

    :ivar asset_id:
    :vartype asset_id: str
    :ivar entity_id:
    :vartype entity_id: str
    :ivar data_items: Dictionary of :code:`<DataItem>`.
    :vartype data_items: dict[str, ~azure.mgmt.machinelearningservices.models.DataItem]
    :ivar data_references:
    :vartype data_references: ~azure.mgmt.machinelearningservices.models.DataReferences
    :ivar should_index:
    :vartype should_index: bool
    :ivar dependencies:
    :vartype dependencies: list[~azure.mgmt.machinelearningservices.models.DependentAsset]
    :ivar intellectual_property_publisher_information:
    :vartype intellectual_property_publisher_information:
     ~azure.mgmt.machinelearningservices.models.IntellectualPropertyPublisherInformation
    """

    _attribute_map = {
        'asset_id': {'key': 'assetId', 'type': 'str'},
        'entity_id': {'key': 'entityId', 'type': 'str'},
        'data_items': {'key': 'dataItems', 'type': '{DataItem}'},
        'data_references': {'key': 'dataReferences', 'type': 'DataReferences'},
        'should_index': {'key': 'shouldIndex', 'type': 'bool'},
        'dependencies': {'key': 'dependencies', 'type': '[DependentAsset]'},
        'intellectual_property_publisher_information': {'key': 'intellectualPropertyPublisherInformation', 'type': 'IntellectualPropertyPublisherInformation'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword asset_id:
        :paramtype asset_id: str
        :keyword entity_id:
        :paramtype entity_id: str
        :keyword data_items: Dictionary of :code:`<DataItem>`.
        :paramtype data_items: dict[str, ~azure.mgmt.machinelearningservices.models.DataItem]
        :keyword data_references:
        :paramtype data_references: ~azure.mgmt.machinelearningservices.models.DataReferences
        :keyword should_index:
        :paramtype should_index: bool
        :keyword dependencies:
        :paramtype dependencies: list[~azure.mgmt.machinelearningservices.models.DependentAsset]
        :keyword intellectual_property_publisher_information:
        :paramtype intellectual_property_publisher_information:
         ~azure.mgmt.machinelearningservices.models.IntellectualPropertyPublisherInformation
        """
        super(AssetDto, self).__init__(**kwargs)
        self.asset_id = kwargs.get('asset_id', None)
        self.entity_id = kwargs.get('entity_id', None)
        self.data_items = kwargs.get('data_items', None)
        self.data_references = kwargs.get('data_references', None)
        self.should_index = kwargs.get('should_index', None)
        self.dependencies = kwargs.get('dependencies', None)
        self.intellectual_property_publisher_information = kwargs.get('intellectual_property_publisher_information', None)


class AssetPaginatedResult(msrest.serialization.Model):
    """AssetPaginatedResult.

    :ivar value:
    :vartype value: list[~azure.mgmt.machinelearningservices.models.Asset]
    :ivar continuation_token:
    :vartype continuation_token: str
    :ivar next_link:
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[Asset]'},
        'continuation_token': {'key': 'continuationToken', 'type': 'str'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword value:
        :paramtype value: list[~azure.mgmt.machinelearningservices.models.Asset]
        :keyword continuation_token:
        :paramtype continuation_token: str
        :keyword next_link:
        :paramtype next_link: str
        """
        super(AssetPaginatedResult, self).__init__(**kwargs)
        self.value = kwargs.get('value', None)
        self.continuation_token = kwargs.get('continuation_token', None)
        self.next_link = kwargs.get('next_link', None)


class BatchGetResolvedUrisDto(msrest.serialization.Model):
    """BatchGetResolvedUrisDto.

    :ivar values:
    :vartype values: list[str]
    """

    _attribute_map = {
        'values': {'key': 'values', 'type': '[str]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword values:
        :paramtype values: list[str]
        """
        super(BatchGetResolvedUrisDto, self).__init__(**kwargs)
        self.values = kwargs.get('values', None)


class BatchModelPathResponseDto(msrest.serialization.Model):
    """BatchModelPathResponseDto.

    :ivar values: Dictionary of :code:`<ModelPathResponseDto>`.
    :vartype values: dict[str, ~azure.mgmt.machinelearningservices.models.ModelPathResponseDto]
    """

    _attribute_map = {
        'values': {'key': 'values', 'type': '{ModelPathResponseDto}'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword values: Dictionary of :code:`<ModelPathResponseDto>`.
        :paramtype values: dict[str, ~azure.mgmt.machinelearningservices.models.ModelPathResponseDto]
        """
        super(BatchModelPathResponseDto, self).__init__(**kwargs)
        self.values = kwargs.get('values', None)


class BlobReference(msrest.serialization.Model):
    """BlobReference.

    :ivar blob_uri:
    :vartype blob_uri: str
    :ivar storage_account_arm_id:
    :vartype storage_account_arm_id: str
    """

    _attribute_map = {
        'blob_uri': {'key': 'blobUri', 'type': 'str'},
        'storage_account_arm_id': {'key': 'storageAccountArmId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword blob_uri:
        :paramtype blob_uri: str
        :keyword storage_account_arm_id:
        :paramtype storage_account_arm_id: str
        """
        super(BlobReference, self).__init__(**kwargs)
        self.blob_uri = kwargs.get('blob_uri', None)
        self.storage_account_arm_id = kwargs.get('storage_account_arm_id', None)


class BlobReferenceForConsumptionDto(msrest.serialization.Model):
    """BlobReferenceForConsumptionDto.

    :ivar blob_uri:
    :vartype blob_uri: str
    :ivar storage_account_arm_id:
    :vartype storage_account_arm_id: str
    :ivar credential:
    :vartype credential: ~azure.mgmt.machinelearningservices.models.DataReferenceCredentialDto
    """

    _attribute_map = {
        'blob_uri': {'key': 'blobUri', 'type': 'str'},
        'storage_account_arm_id': {'key': 'storageAccountArmId', 'type': 'str'},
        'credential': {'key': 'credential', 'type': 'DataReferenceCredentialDto'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword blob_uri:
        :paramtype blob_uri: str
        :keyword storage_account_arm_id:
        :paramtype storage_account_arm_id: str
        :keyword credential:
        :paramtype credential: ~azure.mgmt.machinelearningservices.models.DataReferenceCredentialDto
        """
        super(BlobReferenceForConsumptionDto, self).__init__(**kwargs)
        self.blob_uri = kwargs.get('blob_uri', None)
        self.storage_account_arm_id = kwargs.get('storage_account_arm_id', None)
        self.credential = kwargs.get('credential', None)


class ContainerResourceRequirements(msrest.serialization.Model):
    """ContainerResourceRequirements.

    :ivar cpu:
    :vartype cpu: float
    :ivar cpu_limit:
    :vartype cpu_limit: float
    :ivar memory_in_gb:
    :vartype memory_in_gb: float
    :ivar memory_in_gb_limit:
    :vartype memory_in_gb_limit: float
    :ivar gpu_enabled:
    :vartype gpu_enabled: bool
    :ivar gpu:
    :vartype gpu: int
    :ivar fpga:
    :vartype fpga: int
    """

    _attribute_map = {
        'cpu': {'key': 'cpu', 'type': 'float'},
        'cpu_limit': {'key': 'cpuLimit', 'type': 'float'},
        'memory_in_gb': {'key': 'memoryInGB', 'type': 'float'},
        'memory_in_gb_limit': {'key': 'memoryInGBLimit', 'type': 'float'},
        'gpu_enabled': {'key': 'gpuEnabled', 'type': 'bool'},
        'gpu': {'key': 'gpu', 'type': 'int'},
        'fpga': {'key': 'fpga', 'type': 'int'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword cpu:
        :paramtype cpu: float
        :keyword cpu_limit:
        :paramtype cpu_limit: float
        :keyword memory_in_gb:
        :paramtype memory_in_gb: float
        :keyword memory_in_gb_limit:
        :paramtype memory_in_gb_limit: float
        :keyword gpu_enabled:
        :paramtype gpu_enabled: bool
        :keyword gpu:
        :paramtype gpu: int
        :keyword fpga:
        :paramtype fpga: int
        """
        super(ContainerResourceRequirements, self).__init__(**kwargs)
        self.cpu = kwargs.get('cpu', None)
        self.cpu_limit = kwargs.get('cpu_limit', None)
        self.memory_in_gb = kwargs.get('memory_in_gb', None)
        self.memory_in_gb_limit = kwargs.get('memory_in_gb_limit', None)
        self.gpu_enabled = kwargs.get('gpu_enabled', None)
        self.gpu = kwargs.get('gpu', None)
        self.fpga = kwargs.get('fpga', None)


class CreatedBy(msrest.serialization.Model):
    """CreatedBy.

    :ivar user_object_id:
    :vartype user_object_id: str
    :ivar user_tenant_id:
    :vartype user_tenant_id: str
    :ivar user_name:
    :vartype user_name: str
    """

    _attribute_map = {
        'user_object_id': {'key': 'userObjectId', 'type': 'str'},
        'user_tenant_id': {'key': 'userTenantId', 'type': 'str'},
        'user_name': {'key': 'userName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword user_object_id:
        :paramtype user_object_id: str
        :keyword user_tenant_id:
        :paramtype user_tenant_id: str
        :keyword user_name:
        :paramtype user_name: str
        """
        super(CreatedBy, self).__init__(**kwargs)
        self.user_object_id = kwargs.get('user_object_id', None)
        self.user_tenant_id = kwargs.get('user_tenant_id', None)
        self.user_name = kwargs.get('user_name', None)


class CreateUnregisteredInputModelDto(msrest.serialization.Model):
    """CreateUnregisteredInputModelDto.

    :ivar run_id:
    :vartype run_id: str
    :ivar input_name:
    :vartype input_name: str
    :ivar path:
    :vartype path: str
    :ivar type:
    :vartype type: str
    """

    _attribute_map = {
        'run_id': {'key': 'runId', 'type': 'str'},
        'input_name': {'key': 'inputName', 'type': 'str'},
        'path': {'key': 'path', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword run_id:
        :paramtype run_id: str
        :keyword input_name:
        :paramtype input_name: str
        :keyword path:
        :paramtype path: str
        :keyword type:
        :paramtype type: str
        """
        super(CreateUnregisteredInputModelDto, self).__init__(**kwargs)
        self.run_id = kwargs.get('run_id', None)
        self.input_name = kwargs.get('input_name', None)
        self.path = kwargs.get('path', None)
        self.type = kwargs.get('type', None)


class CreateUnregisteredOutputModelDto(msrest.serialization.Model):
    """CreateUnregisteredOutputModelDto.

    :ivar run_id:
    :vartype run_id: str
    :ivar output_name:
    :vartype output_name: str
    :ivar path:
    :vartype path: str
    :ivar type:
    :vartype type: str
    """

    _attribute_map = {
        'run_id': {'key': 'runId', 'type': 'str'},
        'output_name': {'key': 'outputName', 'type': 'str'},
        'path': {'key': 'path', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword run_id:
        :paramtype run_id: str
        :keyword output_name:
        :paramtype output_name: str
        :keyword path:
        :paramtype path: str
        :keyword type:
        :paramtype type: str
        """
        super(CreateUnregisteredOutputModelDto, self).__init__(**kwargs)
        self.run_id = kwargs.get('run_id', None)
        self.output_name = kwargs.get('output_name', None)
        self.path = kwargs.get('path', None)
        self.type = kwargs.get('type', None)


class CreationContext(msrest.serialization.Model):
    """CreationContext.

    :ivar additional_properties: Unmatched properties from the message are deserialized to this
     collection.
    :vartype additional_properties: dict[str, any]
    :ivar created_time:
    :vartype created_time: ~datetime.datetime
    :ivar created_by:
    :vartype created_by: ~azure.mgmt.machinelearningservices.models.CreatedBy
    :ivar creation_source:
    :vartype creation_source: str
    """

    _attribute_map = {
        'additional_properties': {'key': '', 'type': '{object}'},
        'created_time': {'key': 'createdTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'CreatedBy'},
        'creation_source': {'key': 'creationSource', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword additional_properties: Unmatched properties from the message are deserialized to this
         collection.
        :paramtype additional_properties: dict[str, any]
        :keyword created_time:
        :paramtype created_time: ~datetime.datetime
        :keyword created_by:
        :paramtype created_by: ~azure.mgmt.machinelearningservices.models.CreatedBy
        :keyword creation_source:
        :paramtype creation_source: str
        """
        super(CreationContext, self).__init__(**kwargs)
        self.additional_properties = kwargs.get('additional_properties', None)
        self.created_time = kwargs.get('created_time', None)
        self.created_by = kwargs.get('created_by', None)
        self.creation_source = kwargs.get('creation_source', None)


class DataItem(msrest.serialization.Model):
    """DataItem.

    :ivar data: Anything.
    :vartype data: any
    """

    _attribute_map = {
        'data': {'key': 'data', 'type': 'object'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword data: Anything.
        :paramtype data: any
        """
        super(DataItem, self).__init__(**kwargs)
        self.data = kwargs.get('data', None)


class DataReferenceCredentialDto(msrest.serialization.Model):
    """DataReferenceCredentialDto.

    :ivar additional_properties: Unmatched properties from the message are deserialized to this
     collection.
    :vartype additional_properties: dict[str, any]
    :ivar credential_type:
    :vartype credential_type: str
    """

    _attribute_map = {
        'additional_properties': {'key': '', 'type': '{object}'},
        'credential_type': {'key': 'credentialType', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword additional_properties: Unmatched properties from the message are deserialized to this
         collection.
        :paramtype additional_properties: dict[str, any]
        :keyword credential_type:
        :paramtype credential_type: str
        """
        super(DataReferenceCredentialDto, self).__init__(**kwargs)
        self.additional_properties = kwargs.get('additional_properties', None)
        self.credential_type = kwargs.get('credential_type', None)


class DataReferences(msrest.serialization.Model):
    """DataReferences.

    :ivar blob_references: Dictionary of :code:`<BlobReference>`.
    :vartype blob_references: dict[str, ~azure.mgmt.machinelearningservices.models.BlobReference]
    :ivar image_registry_references: Dictionary of :code:`<ImageReference>`.
    :vartype image_registry_references: dict[str,
     ~azure.mgmt.machinelearningservices.models.ImageReference]
    """

    _attribute_map = {
        'blob_references': {'key': 'blobReferences', 'type': '{BlobReference}'},
        'image_registry_references': {'key': 'imageRegistryReferences', 'type': '{ImageReference}'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword blob_references: Dictionary of :code:`<BlobReference>`.
        :paramtype blob_references: dict[str, ~azure.mgmt.machinelearningservices.models.BlobReference]
        :keyword image_registry_references: Dictionary of :code:`<ImageReference>`.
        :paramtype image_registry_references: dict[str,
         ~azure.mgmt.machinelearningservices.models.ImageReference]
        """
        super(DataReferences, self).__init__(**kwargs)
        self.blob_references = kwargs.get('blob_references', None)
        self.image_registry_references = kwargs.get('image_registry_references', None)


class DataReferencesForConsumptionDto(msrest.serialization.Model):
    """DataReferencesForConsumptionDto.

    :ivar blob_references: Dictionary of :code:`<BlobReferenceForConsumptionDto>`.
    :vartype blob_references: dict[str,
     ~azure.mgmt.machinelearningservices.models.BlobReferenceForConsumptionDto]
    :ivar image_registry_references: Dictionary of :code:`<ImageReferenceForConsumptionDto>`.
    :vartype image_registry_references: dict[str,
     ~azure.mgmt.machinelearningservices.models.ImageReferenceForConsumptionDto]
    """

    _attribute_map = {
        'blob_references': {'key': 'blobReferences', 'type': '{BlobReferenceForConsumptionDto}'},
        'image_registry_references': {'key': 'imageRegistryReferences', 'type': '{ImageReferenceForConsumptionDto}'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword blob_references: Dictionary of :code:`<BlobReferenceForConsumptionDto>`.
        :paramtype blob_references: dict[str,
         ~azure.mgmt.machinelearningservices.models.BlobReferenceForConsumptionDto]
        :keyword image_registry_references: Dictionary of :code:`<ImageReferenceForConsumptionDto>`.
        :paramtype image_registry_references: dict[str,
         ~azure.mgmt.machinelearningservices.models.ImageReferenceForConsumptionDto]
        """
        super(DataReferencesForConsumptionDto, self).__init__(**kwargs)
        self.blob_references = kwargs.get('blob_references', None)
        self.image_registry_references = kwargs.get('image_registry_references', None)


class DatasetReference(msrest.serialization.Model):
    """DatasetReference.

    :ivar name:
    :vartype name: str
    :ivar id:
    :vartype id: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword name:
        :paramtype name: str
        :keyword id:
        :paramtype id: str
        """
        super(DatasetReference, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.id = kwargs.get('id', None)


class DependencyMapDto(msrest.serialization.Model):
    """DependencyMapDto.

    :ivar dependencies:
    :vartype dependencies: list[~azure.mgmt.machinelearningservices.models.DependencyMapItemDto]
    """

    _attribute_map = {
        'dependencies': {'key': 'dependencies', 'type': '[DependencyMapItemDto]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword dependencies:
        :paramtype dependencies: list[~azure.mgmt.machinelearningservices.models.DependencyMapItemDto]
        """
        super(DependencyMapDto, self).__init__(**kwargs)
        self.dependencies = kwargs.get('dependencies', None)


class DependencyMapItemDto(msrest.serialization.Model):
    """DependencyMapItemDto.

    :ivar source_id:
    :vartype source_id: str
    :ivar destination_id:
    :vartype destination_id: str
    """

    _attribute_map = {
        'source_id': {'key': 'sourceId', 'type': 'str'},
        'destination_id': {'key': 'destinationId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword source_id:
        :paramtype source_id: str
        :keyword destination_id:
        :paramtype destination_id: str
        """
        super(DependencyMapItemDto, self).__init__(**kwargs)
        self.source_id = kwargs.get('source_id', None)
        self.destination_id = kwargs.get('destination_id', None)


class DependentAsset(msrest.serialization.Model):
    """DependentAsset.

    :ivar asset_id:
    :vartype asset_id: str
    """

    _attribute_map = {
        'asset_id': {'key': 'assetId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword asset_id:
        :paramtype asset_id: str
        """
        super(DependentAsset, self).__init__(**kwargs)
        self.asset_id = kwargs.get('asset_id', None)


class DependentEntitiesDto(msrest.serialization.Model):
    """DependentEntitiesDto.

    :ivar asset_id:
    :vartype asset_id: str
    :ivar dependencies:
    :vartype dependencies: list[~azure.mgmt.machinelearningservices.models.DependentAsset]
    """

    _attribute_map = {
        'asset_id': {'key': 'assetId', 'type': 'str'},
        'dependencies': {'key': 'dependencies', 'type': '[DependentAsset]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword asset_id:
        :paramtype asset_id: str
        :keyword dependencies:
        :paramtype dependencies: list[~azure.mgmt.machinelearningservices.models.DependentAsset]
        """
        super(DependentEntitiesDto, self).__init__(**kwargs)
        self.asset_id = kwargs.get('asset_id', None)
        self.dependencies = kwargs.get('dependencies', None)


class ErrorResponse(msrest.serialization.Model):
    """ErrorResponse.

    :ivar code:
    :vartype code: str
    :ivar status_code:
    :vartype status_code: int
    :ivar message:
    :vartype message: str
    :ivar target:
    :vartype target: str
    :ivar details:
    :vartype details: list[~azure.mgmt.machinelearningservices.models.InnerErrorDetails]
    :ivar correlation: Dictionary of :code:`<string>`.
    :vartype correlation: dict[str, str]
    """

    _attribute_map = {
        'code': {'key': 'code', 'type': 'str'},
        'status_code': {'key': 'statusCode', 'type': 'int'},
        'message': {'key': 'message', 'type': 'str'},
        'target': {'key': 'target', 'type': 'str'},
        'details': {'key': 'details', 'type': '[InnerErrorDetails]'},
        'correlation': {'key': 'correlation', 'type': '{str}'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword code:
        :paramtype code: str
        :keyword status_code:
        :paramtype status_code: int
        :keyword message:
        :paramtype message: str
        :keyword target:
        :paramtype target: str
        :keyword details:
        :paramtype details: list[~azure.mgmt.machinelearningservices.models.InnerErrorDetails]
        :keyword correlation: Dictionary of :code:`<string>`.
        :paramtype correlation: dict[str, str]
        """
        super(ErrorResponse, self).__init__(**kwargs)
        self.code = kwargs.get('code', None)
        self.status_code = kwargs.get('status_code', None)
        self.message = kwargs.get('message', None)
        self.target = kwargs.get('target', None)
        self.details = kwargs.get('details', None)
        self.correlation = kwargs.get('correlation', None)


class ExtensiveModel(msrest.serialization.Model):
    """ExtensiveModel.

    :ivar model:
    :vartype model: ~azure.mgmt.machinelearningservices.models.Model
    :ivar service_list:
    :vartype service_list: list[~azure.mgmt.machinelearningservices.models.ServiceResponseBase]
    :ivar asset_list:
    :vartype asset_list: list[~azure.mgmt.machinelearningservices.models.Asset]
    """

    _attribute_map = {
        'model': {'key': 'Model', 'type': 'Model'},
        'service_list': {'key': 'ServiceList', 'type': '[ServiceResponseBase]'},
        'asset_list': {'key': 'AssetList', 'type': '[Asset]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword model:
        :paramtype model: ~azure.mgmt.machinelearningservices.models.Model
        :keyword service_list:
        :paramtype service_list: list[~azure.mgmt.machinelearningservices.models.ServiceResponseBase]
        :keyword asset_list:
        :paramtype asset_list: list[~azure.mgmt.machinelearningservices.models.Asset]
        """
        super(ExtensiveModel, self).__init__(**kwargs)
        self.model = kwargs.get('model', None)
        self.service_list = kwargs.get('service_list', None)
        self.asset_list = kwargs.get('asset_list', None)


class FeedIndexEntityDto(msrest.serialization.Model):
    """FeedIndexEntityDto.

    :ivar index_entity:
    :vartype index_entity: ~azure.mgmt.machinelearningservices.models.IndexEntity
    :ivar schema_id:
    :vartype schema_id: str
    :ivar entity_schema: Anything.
    :vartype entity_schema: any
    """

    _attribute_map = {
        'index_entity': {'key': 'indexEntity', 'type': 'IndexEntity'},
        'schema_id': {'key': 'schemaId', 'type': 'str'},
        'entity_schema': {'key': 'entitySchema', 'type': 'object'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword index_entity:
        :paramtype index_entity: ~azure.mgmt.machinelearningservices.models.IndexEntity
        :keyword schema_id:
        :paramtype schema_id: str
        :keyword entity_schema: Anything.
        :paramtype entity_schema: any
        """
        super(FeedIndexEntityDto, self).__init__(**kwargs)
        self.index_entity = kwargs.get('index_entity', None)
        self.schema_id = kwargs.get('schema_id', None)
        self.entity_schema = kwargs.get('entity_schema', None)


class FeedIndexEntityRequestDto(msrest.serialization.Model):
    """FeedIndexEntityRequestDto.

    :ivar feed_entity:
    :vartype feed_entity: ~azure.mgmt.machinelearningservices.models.AssetDto
    :ivar label_to_version_mapping: Dictionary of :code:`<string>`.
    :vartype label_to_version_mapping: dict[str, str]
    """

    _attribute_map = {
        'feed_entity': {'key': 'feedEntity', 'type': 'AssetDto'},
        'label_to_version_mapping': {'key': 'labelToVersionMapping', 'type': '{str}'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword feed_entity:
        :paramtype feed_entity: ~azure.mgmt.machinelearningservices.models.AssetDto
        :keyword label_to_version_mapping: Dictionary of :code:`<string>`.
        :paramtype label_to_version_mapping: dict[str, str]
        """
        super(FeedIndexEntityRequestDto, self).__init__(**kwargs)
        self.feed_entity = kwargs.get('feed_entity', None)
        self.label_to_version_mapping = kwargs.get('label_to_version_mapping', None)


class ImageReference(msrest.serialization.Model):
    """ImageReference.

    :ivar image_registry_reference:
    :vartype image_registry_reference: str
    """

    _attribute_map = {
        'image_registry_reference': {'key': 'imageRegistryReference', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword image_registry_reference:
        :paramtype image_registry_reference: str
        """
        super(ImageReference, self).__init__(**kwargs)
        self.image_registry_reference = kwargs.get('image_registry_reference', None)


class ImageReferenceForConsumptionDto(msrest.serialization.Model):
    """ImageReferenceForConsumptionDto.

    :ivar image_registry_reference:
    :vartype image_registry_reference: str
    :ivar credential:
    :vartype credential: ~azure.mgmt.machinelearningservices.models.DataReferenceCredentialDto
    """

    _attribute_map = {
        'image_registry_reference': {'key': 'imageRegistryReference', 'type': 'str'},
        'credential': {'key': 'credential', 'type': 'DataReferenceCredentialDto'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword image_registry_reference:
        :paramtype image_registry_reference: str
        :keyword credential:
        :paramtype credential: ~azure.mgmt.machinelearningservices.models.DataReferenceCredentialDto
        """
        super(ImageReferenceForConsumptionDto, self).__init__(**kwargs)
        self.image_registry_reference = kwargs.get('image_registry_reference', None)
        self.credential = kwargs.get('credential', None)


class IndexAnnotations(msrest.serialization.Model):
    """IndexAnnotations.

    :ivar additional_properties: Unmatched properties from the message are deserialized to this
     collection.
    :vartype additional_properties: dict[str, any]
    :ivar archived:
    :vartype archived: bool
    :ivar tags: A set of tags. Dictionary of :code:`<string>`.
    :vartype tags: dict[str, str]
    """

    _attribute_map = {
        'additional_properties': {'key': '', 'type': '{object}'},
        'archived': {'key': 'archived', 'type': 'bool'},
        'tags': {'key': 'tags', 'type': '{str}'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword additional_properties: Unmatched properties from the message are deserialized to this
         collection.
        :paramtype additional_properties: dict[str, any]
        :keyword archived:
        :paramtype archived: bool
        :keyword tags: A set of tags. Dictionary of :code:`<string>`.
        :paramtype tags: dict[str, str]
        """
        super(IndexAnnotations, self).__init__(**kwargs)
        self.additional_properties = kwargs.get('additional_properties', None)
        self.archived = kwargs.get('archived', None)
        self.tags = kwargs.get('tags', None)


class IndexEntity(msrest.serialization.Model):
    """IndexEntity.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar schema_id:
    :vartype schema_id: str
    :ivar entity_id:
    :vartype entity_id: str
    :ivar kind: Possible values include: "Invalid", "LineageRoot", "Versioned", "Unversioned".
    :vartype kind: str or ~azure.mgmt.machinelearningservices.models.EntityKind
    :ivar annotations:
    :vartype annotations: ~azure.mgmt.machinelearningservices.models.IndexAnnotations
    :ivar properties:
    :vartype properties: ~azure.mgmt.machinelearningservices.models.IndexProperties
    :ivar internal: Dictionary of :code:`<any>`.
    :vartype internal: dict[str, any]
    :ivar update_sequence:
    :vartype update_sequence: long
    :ivar type:
    :vartype type: str
    :ivar version:
    :vartype version: str
    :ivar entity_container_id:
    :vartype entity_container_id: str
    :ivar entity_object_id:
    :vartype entity_object_id: str
    :ivar resource_type:
    :vartype resource_type: str
    :ivar relationships:
    :vartype relationships: list[~azure.mgmt.machinelearningservices.models.Relationship]
    :ivar asset_id:
    :vartype asset_id: str
    """

    _validation = {
        'version': {'readonly': True},
        'entity_container_id': {'readonly': True},
        'entity_object_id': {'readonly': True},
        'resource_type': {'readonly': True},
    }

    _attribute_map = {
        'schema_id': {'key': 'schemaId', 'type': 'str'},
        'entity_id': {'key': 'entityId', 'type': 'str'},
        'kind': {'key': 'kind', 'type': 'str'},
        'annotations': {'key': 'annotations', 'type': 'IndexAnnotations'},
        'properties': {'key': 'properties', 'type': 'IndexProperties'},
        'internal': {'key': 'internal', 'type': '{object}'},
        'update_sequence': {'key': 'updateSequence', 'type': 'long'},
        'type': {'key': 'type', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
        'entity_container_id': {'key': 'entityContainerId', 'type': 'str'},
        'entity_object_id': {'key': 'entityObjectId', 'type': 'str'},
        'resource_type': {'key': 'resourceType', 'type': 'str'},
        'relationships': {'key': 'relationships', 'type': '[Relationship]'},
        'asset_id': {'key': 'assetId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword schema_id:
        :paramtype schema_id: str
        :keyword entity_id:
        :paramtype entity_id: str
        :keyword kind: Possible values include: "Invalid", "LineageRoot", "Versioned", "Unversioned".
        :paramtype kind: str or ~azure.mgmt.machinelearningservices.models.EntityKind
        :keyword annotations:
        :paramtype annotations: ~azure.mgmt.machinelearningservices.models.IndexAnnotations
        :keyword properties:
        :paramtype properties: ~azure.mgmt.machinelearningservices.models.IndexProperties
        :keyword internal: Dictionary of :code:`<any>`.
        :paramtype internal: dict[str, any]
        :keyword update_sequence:
        :paramtype update_sequence: long
        :keyword type:
        :paramtype type: str
        :keyword relationships:
        :paramtype relationships: list[~azure.mgmt.machinelearningservices.models.Relationship]
        :keyword asset_id:
        :paramtype asset_id: str
        """
        super(IndexEntity, self).__init__(**kwargs)
        self.schema_id = kwargs.get('schema_id', None)
        self.entity_id = kwargs.get('entity_id', None)
        self.kind = kwargs.get('kind', None)
        self.annotations = kwargs.get('annotations', None)
        self.properties = kwargs.get('properties', None)
        self.internal = kwargs.get('internal', None)
        self.update_sequence = kwargs.get('update_sequence', None)
        self.type = kwargs.get('type', None)
        self.version = None
        self.entity_container_id = None
        self.entity_object_id = None
        self.resource_type = None
        self.relationships = kwargs.get('relationships', None)
        self.asset_id = kwargs.get('asset_id', None)


class IndexProperties(msrest.serialization.Model):
    """IndexProperties.

    :ivar additional_properties: Unmatched properties from the message are deserialized to this
     collection.
    :vartype additional_properties: dict[str, any]
    :ivar creation_context:
    :vartype creation_context: ~azure.mgmt.machinelearningservices.models.CreationContext
    """

    _attribute_map = {
        'additional_properties': {'key': '', 'type': '{object}'},
        'creation_context': {'key': 'creationContext', 'type': 'CreationContext'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword additional_properties: Unmatched properties from the message are deserialized to this
         collection.
        :paramtype additional_properties: dict[str, any]
        :keyword creation_context:
        :paramtype creation_context: ~azure.mgmt.machinelearningservices.models.CreationContext
        """
        super(IndexProperties, self).__init__(**kwargs)
        self.additional_properties = kwargs.get('additional_properties', None)
        self.creation_context = kwargs.get('creation_context', None)


class InnerErrorDetails(msrest.serialization.Model):
    """InnerErrorDetails.

    :ivar code:
    :vartype code: str
    :ivar message:
    :vartype message: str
    :ivar target:
    :vartype target: str
    """

    _attribute_map = {
        'code': {'key': 'code', 'type': 'str'},
        'message': {'key': 'message', 'type': 'str'},
        'target': {'key': 'target', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword code:
        :paramtype code: str
        :keyword message:
        :paramtype message: str
        :keyword target:
        :paramtype target: str
        """
        super(InnerErrorDetails, self).__init__(**kwargs)
        self.code = kwargs.get('code', None)
        self.message = kwargs.get('message', None)
        self.target = kwargs.get('target', None)


class IntellectualPropertyPublisherInformation(msrest.serialization.Model):
    """IntellectualPropertyPublisherInformation.

    :ivar intellectual_property_publisher:
    :vartype intellectual_property_publisher: str
    """

    _attribute_map = {
        'intellectual_property_publisher': {'key': 'intellectualPropertyPublisher', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword intellectual_property_publisher:
        :paramtype intellectual_property_publisher: str
        """
        super(IntellectualPropertyPublisherInformation, self).__init__(**kwargs)
        self.intellectual_property_publisher = kwargs.get('intellectual_property_publisher', None)


class ListModelsRequest(msrest.serialization.Model):
    """ListModelsRequest.

    :ivar name:
    :vartype name: str
    :ivar tag:
    :vartype tag: str
    :ivar version:
    :vartype version: str
    :ivar framework:
    :vartype framework: str
    :ivar description:
    :vartype description: str
    :ivar count:
    :vartype count: int
    :ivar offset:
    :vartype offset: int
    :ivar skip_token:
    :vartype skip_token: str
    :ivar tags: A set of tags.
    :vartype tags: str
    :ivar properties:
    :vartype properties: str
    :ivar run_id:
    :vartype run_id: str
    :ivar dataset_id:
    :vartype dataset_id: str
    :ivar order_by: Possible values include: "CreatedAtDesc", "CreatedAtAsc", "UpdatedAtDesc",
     "UpdatedAtAsc".
    :vartype order_by: str or ~azure.mgmt.machinelearningservices.models.OrderString
    :ivar latest_version_only:
    :vartype latest_version_only: bool
    :ivar modified_after:
    :vartype modified_after: ~datetime.datetime
    :ivar modified_before:
    :vartype modified_before: ~datetime.datetime
    :ivar list_view_type: Possible values include: "ActiveOnly", "ArchivedOnly", "All".
    :vartype list_view_type: str or ~azure.mgmt.machinelearningservices.models.ListViewType
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'tag': {'key': 'tag', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
        'framework': {'key': 'framework', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'count': {'key': 'count', 'type': 'int'},
        'offset': {'key': 'offset', 'type': 'int'},
        'skip_token': {'key': 'skipToken', 'type': 'str'},
        'tags': {'key': 'tags', 'type': 'str'},
        'properties': {'key': 'properties', 'type': 'str'},
        'run_id': {'key': 'runId', 'type': 'str'},
        'dataset_id': {'key': 'datasetId', 'type': 'str'},
        'order_by': {'key': 'orderBy', 'type': 'str'},
        'latest_version_only': {'key': 'latestVersionOnly', 'type': 'bool'},
        'modified_after': {'key': 'modifiedAfter', 'type': 'iso-8601'},
        'modified_before': {'key': 'modifiedBefore', 'type': 'iso-8601'},
        'list_view_type': {'key': 'listViewType', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword name:
        :paramtype name: str
        :keyword tag:
        :paramtype tag: str
        :keyword version:
        :paramtype version: str
        :keyword framework:
        :paramtype framework: str
        :keyword description:
        :paramtype description: str
        :keyword count:
        :paramtype count: int
        :keyword offset:
        :paramtype offset: int
        :keyword skip_token:
        :paramtype skip_token: str
        :keyword tags: A set of tags.
        :paramtype tags: str
        :keyword properties:
        :paramtype properties: str
        :keyword run_id:
        :paramtype run_id: str
        :keyword dataset_id:
        :paramtype dataset_id: str
        :keyword order_by: Possible values include: "CreatedAtDesc", "CreatedAtAsc", "UpdatedAtDesc",
         "UpdatedAtAsc".
        :paramtype order_by: str or ~azure.mgmt.machinelearningservices.models.OrderString
        :keyword latest_version_only:
        :paramtype latest_version_only: bool
        :keyword modified_after:
        :paramtype modified_after: ~datetime.datetime
        :keyword modified_before:
        :paramtype modified_before: ~datetime.datetime
        :keyword list_view_type: Possible values include: "ActiveOnly", "ArchivedOnly", "All".
        :paramtype list_view_type: str or ~azure.mgmt.machinelearningservices.models.ListViewType
        """
        super(ListModelsRequest, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.tag = kwargs.get('tag', None)
        self.version = kwargs.get('version', None)
        self.framework = kwargs.get('framework', None)
        self.description = kwargs.get('description', None)
        self.count = kwargs.get('count', None)
        self.offset = kwargs.get('offset', None)
        self.skip_token = kwargs.get('skip_token', None)
        self.tags = kwargs.get('tags', None)
        self.properties = kwargs.get('properties', None)
        self.run_id = kwargs.get('run_id', None)
        self.dataset_id = kwargs.get('dataset_id', None)
        self.order_by = kwargs.get('order_by', None)
        self.latest_version_only = kwargs.get('latest_version_only', None)
        self.modified_after = kwargs.get('modified_after', None)
        self.modified_before = kwargs.get('modified_before', None)
        self.list_view_type = kwargs.get('list_view_type', None)


class Model(msrest.serialization.Model):
    """Model.

    All required parameters must be populated in order to send to Azure.

    :ivar id:
    :vartype id: str
    :ivar name: Required.
    :vartype name: str
    :ivar framework:
    :vartype framework: str
    :ivar framework_version:
    :vartype framework_version: str
    :ivar version:
    :vartype version: long
    :ivar tags: A set of tags.
    :vartype tags: list[str]
    :ivar datasets:
    :vartype datasets: list[~azure.mgmt.machinelearningservices.models.DatasetReference]
    :ivar url:
    :vartype url: str
    :ivar mime_type: Required.
    :vartype mime_type: str
    :ivar description:
    :vartype description: str
    :ivar created_time:
    :vartype created_time: ~datetime.datetime
    :ivar modified_time:
    :vartype modified_time: ~datetime.datetime
    :ivar unpack:
    :vartype unpack: bool
    :ivar parent_model_id:
    :vartype parent_model_id: str
    :ivar run_id:
    :vartype run_id: str
    :ivar experiment_name:
    :vartype experiment_name: str
    :ivar kv_tags: Dictionary of :code:`<string>`.
    :vartype kv_tags: dict[str, str]
    :ivar properties: Dictionary of :code:`<string>`.
    :vartype properties: dict[str, str]
    :ivar derived_model_ids:
    :vartype derived_model_ids: list[str]
    :ivar inputs_schema:
    :vartype inputs_schema: list[~azure.mgmt.machinelearningservices.models.ModelSchema]
    :ivar outputs_schema:
    :vartype outputs_schema: list[~azure.mgmt.machinelearningservices.models.ModelSchema]
    :ivar sample_input_data:
    :vartype sample_input_data: str
    :ivar sample_output_data:
    :vartype sample_output_data: str
    :ivar resource_requirements:
    :vartype resource_requirements:
     ~azure.mgmt.machinelearningservices.models.ContainerResourceRequirements
    :ivar created_by:
    :vartype created_by: ~azure.mgmt.machinelearningservices.models.User
    :ivar modified_by:
    :vartype modified_by: ~azure.mgmt.machinelearningservices.models.User
    :ivar flavors: Dictionary of
     <components·8urbg9·schemas·model·properties·flavors·additionalproperties>.
    :vartype flavors: dict[str, dict[str, str]]
    :ivar model_format: Possible values include: "CUSTOM", "MLFLOW", "TRITON", "PRESETS".
    :vartype model_format: str or ~azure.mgmt.machinelearningservices.models.ModelFormatEnum
    :ivar stage:
    :vartype stage: str
    :ivar model_container_id:
    :vartype model_container_id: str
    :ivar mms_id:
    :vartype mms_id: str
    :ivar default_deployment_settings:
    :vartype default_deployment_settings:
     ~azure.mgmt.machinelearningservices.models.ModelDeploymentSettings
    :ivar is_anonymous:
    :vartype is_anonymous: bool
    :ivar is_archived:
    :vartype is_archived: bool
    :ivar is_registered:
    :vartype is_registered: bool
    :ivar data_path:
    :vartype data_path: str
    :ivar model_type:
    :vartype model_type: str
    :ivar asset_id:
    :vartype asset_id: str
    """

    _validation = {
        'name': {'required': True},
        'mime_type': {'required': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'framework': {'key': 'framework', 'type': 'str'},
        'framework_version': {'key': 'frameworkVersion', 'type': 'str'},
        'version': {'key': 'version', 'type': 'long'},
        'tags': {'key': 'tags', 'type': '[str]'},
        'datasets': {'key': 'datasets', 'type': '[DatasetReference]'},
        'url': {'key': 'url', 'type': 'str'},
        'mime_type': {'key': 'mimeType', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'created_time': {'key': 'createdTime', 'type': 'iso-8601'},
        'modified_time': {'key': 'modifiedTime', 'type': 'iso-8601'},
        'unpack': {'key': 'unpack', 'type': 'bool'},
        'parent_model_id': {'key': 'parentModelId', 'type': 'str'},
        'run_id': {'key': 'runId', 'type': 'str'},
        'experiment_name': {'key': 'experimentName', 'type': 'str'},
        'kv_tags': {'key': 'kvTags', 'type': '{str}'},
        'properties': {'key': 'properties', 'type': '{str}'},
        'derived_model_ids': {'key': 'derivedModelIds', 'type': '[str]'},
        'inputs_schema': {'key': 'inputsSchema', 'type': '[ModelSchema]'},
        'outputs_schema': {'key': 'outputsSchema', 'type': '[ModelSchema]'},
        'sample_input_data': {'key': 'sampleInputData', 'type': 'str'},
        'sample_output_data': {'key': 'sampleOutputData', 'type': 'str'},
        'resource_requirements': {'key': 'resourceRequirements', 'type': 'ContainerResourceRequirements'},
        'created_by': {'key': 'createdBy', 'type': 'User'},
        'modified_by': {'key': 'modifiedBy', 'type': 'User'},
        'flavors': {'key': 'flavors', 'type': '{{str}}'},
        'model_format': {'key': 'modelFormat', 'type': 'str'},
        'stage': {'key': 'stage', 'type': 'str'},
        'model_container_id': {'key': 'modelContainerId', 'type': 'str'},
        'mms_id': {'key': 'mmsId', 'type': 'str'},
        'default_deployment_settings': {'key': 'defaultDeploymentSettings', 'type': 'ModelDeploymentSettings'},
        'is_anonymous': {'key': 'isAnonymous', 'type': 'bool'},
        'is_archived': {'key': 'isArchived', 'type': 'bool'},
        'is_registered': {'key': 'isRegistered', 'type': 'bool'},
        'data_path': {'key': 'dataPath', 'type': 'str'},
        'model_type': {'key': 'modelType', 'type': 'str'},
        'asset_id': {'key': 'assetId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword id:
        :paramtype id: str
        :keyword name: Required.
        :paramtype name: str
        :keyword framework:
        :paramtype framework: str
        :keyword framework_version:
        :paramtype framework_version: str
        :keyword version:
        :paramtype version: long
        :keyword tags: A set of tags.
        :paramtype tags: list[str]
        :keyword datasets:
        :paramtype datasets: list[~azure.mgmt.machinelearningservices.models.DatasetReference]
        :keyword url:
        :paramtype url: str
        :keyword mime_type: Required.
        :paramtype mime_type: str
        :keyword description:
        :paramtype description: str
        :keyword created_time:
        :paramtype created_time: ~datetime.datetime
        :keyword modified_time:
        :paramtype modified_time: ~datetime.datetime
        :keyword unpack:
        :paramtype unpack: bool
        :keyword parent_model_id:
        :paramtype parent_model_id: str
        :keyword run_id:
        :paramtype run_id: str
        :keyword experiment_name:
        :paramtype experiment_name: str
        :keyword kv_tags: Dictionary of :code:`<string>`.
        :paramtype kv_tags: dict[str, str]
        :keyword properties: Dictionary of :code:`<string>`.
        :paramtype properties: dict[str, str]
        :keyword derived_model_ids:
        :paramtype derived_model_ids: list[str]
        :keyword inputs_schema:
        :paramtype inputs_schema: list[~azure.mgmt.machinelearningservices.models.ModelSchema]
        :keyword outputs_schema:
        :paramtype outputs_schema: list[~azure.mgmt.machinelearningservices.models.ModelSchema]
        :keyword sample_input_data:
        :paramtype sample_input_data: str
        :keyword sample_output_data:
        :paramtype sample_output_data: str
        :keyword resource_requirements:
        :paramtype resource_requirements:
         ~azure.mgmt.machinelearningservices.models.ContainerResourceRequirements
        :keyword created_by:
        :paramtype created_by: ~azure.mgmt.machinelearningservices.models.User
        :keyword modified_by:
        :paramtype modified_by: ~azure.mgmt.machinelearningservices.models.User
        :keyword flavors: Dictionary of
         <components·8urbg9·schemas·model·properties·flavors·additionalproperties>.
        :paramtype flavors: dict[str, dict[str, str]]
        :keyword model_format: Possible values include: "CUSTOM", "MLFLOW", "TRITON", "PRESETS".
        :paramtype model_format: str or ~azure.mgmt.machinelearningservices.models.ModelFormatEnum
        :keyword stage:
        :paramtype stage: str
        :keyword model_container_id:
        :paramtype model_container_id: str
        :keyword mms_id:
        :paramtype mms_id: str
        :keyword default_deployment_settings:
        :paramtype default_deployment_settings:
         ~azure.mgmt.machinelearningservices.models.ModelDeploymentSettings
        :keyword is_anonymous:
        :paramtype is_anonymous: bool
        :keyword is_archived:
        :paramtype is_archived: bool
        :keyword is_registered:
        :paramtype is_registered: bool
        :keyword data_path:
        :paramtype data_path: str
        :keyword model_type:
        :paramtype model_type: str
        :keyword asset_id:
        :paramtype asset_id: str
        """
        super(Model, self).__init__(**kwargs)
        self.id = kwargs.get('id', None)
        self.name = kwargs['name']
        self.framework = kwargs.get('framework', None)
        self.framework_version = kwargs.get('framework_version', None)
        self.version = kwargs.get('version', None)
        self.tags = kwargs.get('tags', None)
        self.datasets = kwargs.get('datasets', None)
        self.url = kwargs.get('url', None)
        self.mime_type = kwargs['mime_type']
        self.description = kwargs.get('description', None)
        self.created_time = kwargs.get('created_time', None)
        self.modified_time = kwargs.get('modified_time', None)
        self.unpack = kwargs.get('unpack', None)
        self.parent_model_id = kwargs.get('parent_model_id', None)
        self.run_id = kwargs.get('run_id', None)
        self.experiment_name = kwargs.get('experiment_name', None)
        self.kv_tags = kwargs.get('kv_tags', None)
        self.properties = kwargs.get('properties', None)
        self.derived_model_ids = kwargs.get('derived_model_ids', None)
        self.inputs_schema = kwargs.get('inputs_schema', None)
        self.outputs_schema = kwargs.get('outputs_schema', None)
        self.sample_input_data = kwargs.get('sample_input_data', None)
        self.sample_output_data = kwargs.get('sample_output_data', None)
        self.resource_requirements = kwargs.get('resource_requirements', None)
        self.created_by = kwargs.get('created_by', None)
        self.modified_by = kwargs.get('modified_by', None)
        self.flavors = kwargs.get('flavors', None)
        self.model_format = kwargs.get('model_format', None)
        self.stage = kwargs.get('stage', None)
        self.model_container_id = kwargs.get('model_container_id', None)
        self.mms_id = kwargs.get('mms_id', None)
        self.default_deployment_settings = kwargs.get('default_deployment_settings', None)
        self.is_anonymous = kwargs.get('is_anonymous', None)
        self.is_archived = kwargs.get('is_archived', None)
        self.is_registered = kwargs.get('is_registered', None)
        self.data_path = kwargs.get('data_path', None)
        self.model_type = kwargs.get('model_type', None)
        self.asset_id = kwargs.get('asset_id', None)


class ModelBatchDto(msrest.serialization.Model):
    """ModelBatchDto.

    :ivar model_ids:
    :vartype model_ids: list[str]
    """

    _attribute_map = {
        'model_ids': {'key': 'modelIds', 'type': '[str]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword model_ids:
        :paramtype model_ids: list[str]
        """
        super(ModelBatchDto, self).__init__(**kwargs)
        self.model_ids = kwargs.get('model_ids', None)


class ModelBatchResponseDto(msrest.serialization.Model):
    """ModelBatchResponseDto.

    :ivar models: Dictionary of :code:`<Model>`.
    :vartype models: dict[str, ~azure.mgmt.machinelearningservices.models.Model]
    """

    _attribute_map = {
        'models': {'key': 'models', 'type': '{Model}'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword models: Dictionary of :code:`<Model>`.
        :paramtype models: dict[str, ~azure.mgmt.machinelearningservices.models.Model]
        """
        super(ModelBatchResponseDto, self).__init__(**kwargs)
        self.models = kwargs.get('models', None)


class ModelContainerRequest(msrest.serialization.Model):
    """ModelContainerRequest.

    All required parameters must be populated in order to send to Azure.

    :ivar name: Required.
    :vartype name: str
    :ivar description:
    :vartype description: str
    :ivar kv_tags: Dictionary of :code:`<string>`.
    :vartype kv_tags: dict[str, str]
    :ivar is_archived:
    :vartype is_archived: bool
    :ivar is_registered:
    :vartype is_registered: bool
    """

    _validation = {
        'name': {'required': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'kv_tags': {'key': 'kvTags', 'type': '{str}'},
        'is_archived': {'key': 'isArchived', 'type': 'bool'},
        'is_registered': {'key': 'isRegistered', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword name: Required.
        :paramtype name: str
        :keyword description:
        :paramtype description: str
        :keyword kv_tags: Dictionary of :code:`<string>`.
        :paramtype kv_tags: dict[str, str]
        :keyword is_archived:
        :paramtype is_archived: bool
        :keyword is_registered:
        :paramtype is_registered: bool
        """
        super(ModelContainerRequest, self).__init__(**kwargs)
        self.name = kwargs['name']
        self.description = kwargs.get('description', None)
        self.kv_tags = kwargs.get('kv_tags', None)
        self.is_archived = kwargs.get('is_archived', None)
        self.is_registered = kwargs.get('is_registered', None)


class ModelDeploymentSettings(msrest.serialization.Model):
    """ModelDeploymentSettings.

    All required parameters must be populated in order to send to Azure.

    :ivar model_format: Required. Possible values include: "CUSTOM", "MLFLOW", "TRITON", "PRESETS".
    :vartype model_format: str or ~azure.mgmt.machinelearningservices.models.ModelFormatEnum
    :ivar model_name:
    :vartype model_name: str
    :ivar model_version:
    :vartype model_version: str
    :ivar model_type:
    :vartype model_type: str
    """

    _validation = {
        'model_format': {'required': True},
    }

    _attribute_map = {
        'model_format': {'key': 'modelFormat', 'type': 'str'},
        'model_name': {'key': 'ModelName', 'type': 'str'},
        'model_version': {'key': 'ModelVersion', 'type': 'str'},
        'model_type': {'key': 'ModelType', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword model_format: Required. Possible values include: "CUSTOM", "MLFLOW", "TRITON",
         "PRESETS".
        :paramtype model_format: str or ~azure.mgmt.machinelearningservices.models.ModelFormatEnum
        :keyword model_name:
        :paramtype model_name: str
        :keyword model_version:
        :paramtype model_version: str
        :keyword model_type:
        :paramtype model_type: str
        """
        super(ModelDeploymentSettings, self).__init__(**kwargs)
        self.model_format = kwargs['model_format']
        self.model_name = kwargs.get('model_name', None)
        self.model_version = kwargs.get('model_version', None)
        self.model_type = kwargs.get('model_type', None)


class ModelListModelsRequestPagedResponse(msrest.serialization.Model):
    """ModelListModelsRequestPagedResponse.

    :ivar value:
    :vartype value: list[~azure.mgmt.machinelearningservices.models.Model]
    :ivar next_link:
    :vartype next_link: str
    :ivar continuation_token:
    :vartype continuation_token: str
    :ivar next_request:
    :vartype next_request: ~azure.mgmt.machinelearningservices.models.ListModelsRequest
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[Model]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
        'continuation_token': {'key': 'continuationToken', 'type': 'str'},
        'next_request': {'key': 'nextRequest', 'type': 'ListModelsRequest'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword value:
        :paramtype value: list[~azure.mgmt.machinelearningservices.models.Model]
        :keyword next_link:
        :paramtype next_link: str
        :keyword continuation_token:
        :paramtype continuation_token: str
        :keyword next_request:
        :paramtype next_request: ~azure.mgmt.machinelearningservices.models.ListModelsRequest
        """
        super(ModelListModelsRequestPagedResponse, self).__init__(**kwargs)
        self.value = kwargs.get('value', None)
        self.next_link = kwargs.get('next_link', None)
        self.continuation_token = kwargs.get('continuation_token', None)
        self.next_request = kwargs.get('next_request', None)


class ModelPagedResponse(msrest.serialization.Model):
    """ModelPagedResponse.

    :ivar value:
    :vartype value: list[~azure.mgmt.machinelearningservices.models.Model]
    :ivar continuation_token:
    :vartype continuation_token: str
    :ivar next_link:
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[Model]'},
        'continuation_token': {'key': 'continuationToken', 'type': 'str'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword value:
        :paramtype value: list[~azure.mgmt.machinelearningservices.models.Model]
        :keyword continuation_token:
        :paramtype continuation_token: str
        :keyword next_link:
        :paramtype next_link: str
        """
        super(ModelPagedResponse, self).__init__(**kwargs)
        self.value = kwargs.get('value', None)
        self.continuation_token = kwargs.get('continuation_token', None)
        self.next_link = kwargs.get('next_link', None)


class ModelPathResponseDto(msrest.serialization.Model):
    """ModelPathResponseDto.

    :ivar path:
    :vartype path: str
    :ivar type:
    :vartype type: str
    """

    _attribute_map = {
        'path': {'key': 'path', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword path:
        :paramtype path: str
        :keyword type:
        :paramtype type: str
        """
        super(ModelPathResponseDto, self).__init__(**kwargs)
        self.path = kwargs.get('path', None)
        self.type = kwargs.get('type', None)


class ModelSchema(msrest.serialization.Model):
    """ModelSchema.

    All required parameters must be populated in order to send to Azure.

    :ivar name: Required.
    :vartype name: str
    :ivar data_type: Possible values include: "undefined", "bool", "uint8", "uint16", "uint32",
     "uint64", "int8", "int16", "int32", "int64", "float16", "float32", "float64", "bfloat16",
     "complex64", "complex128", "string".
    :vartype data_type: str or ~azure.mgmt.machinelearningservices.models.ModelSchemaDataType
    :ivar shape:
    :vartype shape: list[int]
    """

    _validation = {
        'name': {'required': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'data_type': {'key': 'dataType', 'type': 'str'},
        'shape': {'key': 'shape', 'type': '[int]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword name: Required.
        :paramtype name: str
        :keyword data_type: Possible values include: "undefined", "bool", "uint8", "uint16", "uint32",
         "uint64", "int8", "int16", "int32", "int64", "float16", "float32", "float64", "bfloat16",
         "complex64", "complex128", "string".
        :paramtype data_type: str or ~azure.mgmt.machinelearningservices.models.ModelSchemaDataType
        :keyword shape:
        :paramtype shape: list[int]
        """
        super(ModelSchema, self).__init__(**kwargs)
        self.name = kwargs['name']
        self.data_type = kwargs.get('data_type', None)
        self.shape = kwargs.get('shape', None)


class ModelSettingsIdentifiers(msrest.serialization.Model):
    """ModelSettingsIdentifiers.

    :ivar model_id:
    :vartype model_id: str
    :ivar engine_id:
    :vartype engine_id: str
    """

    _attribute_map = {
        'model_id': {'key': 'modelId', 'type': 'str'},
        'engine_id': {'key': 'engineId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword model_id:
        :paramtype model_id: str
        :keyword engine_id:
        :paramtype engine_id: str
        """
        super(ModelSettingsIdentifiers, self).__init__(**kwargs)
        self.model_id = kwargs.get('model_id', None)
        self.engine_id = kwargs.get('engine_id', None)


class Operation(msrest.serialization.Model):
    """Operation.

    :ivar value: Anything.
    :vartype value: any
    :ivar path:
    :vartype path: str
    :ivar op:
    :vartype op: str
    :ivar from_property:
    :vartype from_property: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': 'object'},
        'path': {'key': 'path', 'type': 'str'},
        'op': {'key': 'op', 'type': 'str'},
        'from_property': {'key': 'from', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword value: Anything.
        :paramtype value: any
        :keyword path:
        :paramtype path: str
        :keyword op:
        :paramtype op: str
        :keyword from_property:
        :paramtype from_property: str
        """
        super(Operation, self).__init__(**kwargs)
        self.value = kwargs.get('value', None)
        self.path = kwargs.get('path', None)
        self.op = kwargs.get('op', None)
        self.from_property = kwargs.get('from_property', None)


class ProviderFeedEntityRequestDto(msrest.serialization.Model):
    """ProviderFeedEntityRequestDto.

    :ivar source_and_target_asset_ids:
    :vartype source_and_target_asset_ids:
     ~azure.mgmt.machinelearningservices.models.DependencyMapItemDto
    :ivar dependency_map_dto:
    :vartype dependency_map_dto: ~azure.mgmt.machinelearningservices.models.DependencyMapDto
    :ivar label_to_version_mapping: Dictionary of :code:`<string>`.
    :vartype label_to_version_mapping: dict[str, str]
    """

    _attribute_map = {
        'source_and_target_asset_ids': {'key': 'sourceAndTargetAssetIds', 'type': 'DependencyMapItemDto'},
        'dependency_map_dto': {'key': 'dependencyMapDto', 'type': 'DependencyMapDto'},
        'label_to_version_mapping': {'key': 'labelToVersionMapping', 'type': '{str}'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword source_and_target_asset_ids:
        :paramtype source_and_target_asset_ids:
         ~azure.mgmt.machinelearningservices.models.DependencyMapItemDto
        :keyword dependency_map_dto:
        :paramtype dependency_map_dto: ~azure.mgmt.machinelearningservices.models.DependencyMapDto
        :keyword label_to_version_mapping: Dictionary of :code:`<string>`.
        :paramtype label_to_version_mapping: dict[str, str]
        """
        super(ProviderFeedEntityRequestDto, self).__init__(**kwargs)
        self.source_and_target_asset_ids = kwargs.get('source_and_target_asset_ids', None)
        self.dependency_map_dto = kwargs.get('dependency_map_dto', None)
        self.label_to_version_mapping = kwargs.get('label_to_version_mapping', None)


class Relationship(msrest.serialization.Model):
    """Relationship.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar additional_properties: Unmatched properties from the message are deserialized to this
     collection.
    :vartype additional_properties: dict[str, any]
    :ivar relation_type:
    :vartype relation_type: str
    :ivar target_entity_id:
    :vartype target_entity_id: str
    :ivar asset_id:
    :vartype asset_id: str
    :ivar entity_type:
    :vartype entity_type: str
    :ivar direction:
    :vartype direction: str
    :ivar entity_container_id:
    :vartype entity_container_id: str
    """

    _validation = {
        'entity_type': {'readonly': True},
        'entity_container_id': {'readonly': True},
    }

    _attribute_map = {
        'additional_properties': {'key': '', 'type': '{object}'},
        'relation_type': {'key': 'relationType', 'type': 'str'},
        'target_entity_id': {'key': 'targetEntityId', 'type': 'str'},
        'asset_id': {'key': 'assetId', 'type': 'str'},
        'entity_type': {'key': 'entityType', 'type': 'str'},
        'direction': {'key': 'direction', 'type': 'str'},
        'entity_container_id': {'key': 'entityContainerId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword additional_properties: Unmatched properties from the message are deserialized to this
         collection.
        :paramtype additional_properties: dict[str, any]
        :keyword relation_type:
        :paramtype relation_type: str
        :keyword target_entity_id:
        :paramtype target_entity_id: str
        :keyword asset_id:
        :paramtype asset_id: str
        :keyword direction:
        :paramtype direction: str
        """
        super(Relationship, self).__init__(**kwargs)
        self.additional_properties = kwargs.get('additional_properties', None)
        self.relation_type = kwargs.get('relation_type', None)
        self.target_entity_id = kwargs.get('target_entity_id', None)
        self.asset_id = kwargs.get('asset_id', None)
        self.entity_type = None
        self.direction = kwargs.get('direction', None)
        self.entity_container_id = None


class ServiceResponseBase(msrest.serialization.Model):
    """ServiceResponseBase.

    :ivar id:
    :vartype id: str
    :ivar name:
    :vartype name: str
    :ivar description:
    :vartype description: str
    :ivar tags: A set of tags.
    :vartype tags: list[str]
    :ivar kv_tags: Dictionary of :code:`<string>`.
    :vartype kv_tags: dict[str, str]
    :ivar properties: Dictionary of :code:`<string>`.
    :vartype properties: dict[str, str]
    :ivar operation_id:
    :vartype operation_id: str
    :ivar state: Possible values include: "Transitioning", "Healthy", "Unhealthy", "Failed",
     "Unschedulable".
    :vartype state: str or ~azure.mgmt.machinelearningservices.models.WebServiceState
    :ivar created_time:
    :vartype created_time: ~datetime.datetime
    :ivar updated_time:
    :vartype updated_time: ~datetime.datetime
    :ivar error:
    :vartype error: ~azure.mgmt.machinelearningservices.models.ErrorResponse
    :ivar compute_type: Possible values include: "ACS", "FPGA", "ACI", "AKS", "AMLCOMPUTE", "IOT",
     "MIR", "AKSENDPOINT", "MIRSINGLEMODEL", "MIRAMLCOMPUTE", "MIRGA", "AMLARC", "BATCHAMLCOMPUTE",
     "UNKNOWN".
    :vartype compute_type: str or ~azure.mgmt.machinelearningservices.models.ComputeEnvironmentType
    :ivar deployment_type: Possible values include: "GRPCRealtimeEndpoint", "HttpRealtimeEndpoint",
     "Batch".
    :vartype deployment_type: str or ~azure.mgmt.machinelearningservices.models.DeploymentType
    :ivar created_by:
    :vartype created_by: ~azure.mgmt.machinelearningservices.models.User
    :ivar endpoint_name:
    :vartype endpoint_name: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'tags': {'key': 'tags', 'type': '[str]'},
        'kv_tags': {'key': 'kvTags', 'type': '{str}'},
        'properties': {'key': 'properties', 'type': '{str}'},
        'operation_id': {'key': 'operationId', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'created_time': {'key': 'createdTime', 'type': 'iso-8601'},
        'updated_time': {'key': 'updatedTime', 'type': 'iso-8601'},
        'error': {'key': 'error', 'type': 'ErrorResponse'},
        'compute_type': {'key': 'computeType', 'type': 'str'},
        'deployment_type': {'key': 'deploymentType', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'User'},
        'endpoint_name': {'key': 'endpointName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword id:
        :paramtype id: str
        :keyword name:
        :paramtype name: str
        :keyword description:
        :paramtype description: str
        :keyword tags: A set of tags.
        :paramtype tags: list[str]
        :keyword kv_tags: Dictionary of :code:`<string>`.
        :paramtype kv_tags: dict[str, str]
        :keyword properties: Dictionary of :code:`<string>`.
        :paramtype properties: dict[str, str]
        :keyword operation_id:
        :paramtype operation_id: str
        :keyword state: Possible values include: "Transitioning", "Healthy", "Unhealthy", "Failed",
         "Unschedulable".
        :paramtype state: str or ~azure.mgmt.machinelearningservices.models.WebServiceState
        :keyword created_time:
        :paramtype created_time: ~datetime.datetime
        :keyword updated_time:
        :paramtype updated_time: ~datetime.datetime
        :keyword error:
        :paramtype error: ~azure.mgmt.machinelearningservices.models.ErrorResponse
        :keyword compute_type: Possible values include: "ACS", "FPGA", "ACI", "AKS", "AMLCOMPUTE",
         "IOT", "MIR", "AKSENDPOINT", "MIRSINGLEMODEL", "MIRAMLCOMPUTE", "MIRGA", "AMLARC",
         "BATCHAMLCOMPUTE", "UNKNOWN".
        :paramtype compute_type: str or
         ~azure.mgmt.machinelearningservices.models.ComputeEnvironmentType
        :keyword deployment_type: Possible values include: "GRPCRealtimeEndpoint",
         "HttpRealtimeEndpoint", "Batch".
        :paramtype deployment_type: str or ~azure.mgmt.machinelearningservices.models.DeploymentType
        :keyword created_by:
        :paramtype created_by: ~azure.mgmt.machinelearningservices.models.User
        :keyword endpoint_name:
        :paramtype endpoint_name: str
        """
        super(ServiceResponseBase, self).__init__(**kwargs)
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.description = kwargs.get('description', None)
        self.tags = kwargs.get('tags', None)
        self.kv_tags = kwargs.get('kv_tags', None)
        self.properties = kwargs.get('properties', None)
        self.operation_id = kwargs.get('operation_id', None)
        self.state = kwargs.get('state', None)
        self.created_time = kwargs.get('created_time', None)
        self.updated_time = kwargs.get('updated_time', None)
        self.error = kwargs.get('error', None)
        self.compute_type = kwargs.get('compute_type', None)
        self.deployment_type = kwargs.get('deployment_type', None)
        self.created_by = kwargs.get('created_by', None)
        self.endpoint_name = kwargs.get('endpoint_name', None)


class User(msrest.serialization.Model):
    """User.

    :ivar user_object_id:
    :vartype user_object_id: str
    :ivar user_pu_id:
    :vartype user_pu_id: str
    :ivar user_idp:
    :vartype user_idp: str
    :ivar user_alt_sec_id:
    :vartype user_alt_sec_id: str
    :ivar user_iss:
    :vartype user_iss: str
    :ivar user_tenant_id:
    :vartype user_tenant_id: str
    :ivar user_name:
    :vartype user_name: str
    :ivar upn:
    :vartype upn: str
    """

    _attribute_map = {
        'user_object_id': {'key': 'userObjectId', 'type': 'str'},
        'user_pu_id': {'key': 'userPuId', 'type': 'str'},
        'user_idp': {'key': 'userIdp', 'type': 'str'},
        'user_alt_sec_id': {'key': 'userAltSecId', 'type': 'str'},
        'user_iss': {'key': 'userIss', 'type': 'str'},
        'user_tenant_id': {'key': 'userTenantId', 'type': 'str'},
        'user_name': {'key': 'userName', 'type': 'str'},
        'upn': {'key': 'upn', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        :keyword user_object_id:
        :paramtype user_object_id: str
        :keyword user_pu_id:
        :paramtype user_pu_id: str
        :keyword user_idp:
        :paramtype user_idp: str
        :keyword user_alt_sec_id:
        :paramtype user_alt_sec_id: str
        :keyword user_iss:
        :paramtype user_iss: str
        :keyword user_tenant_id:
        :paramtype user_tenant_id: str
        :keyword user_name:
        :paramtype user_name: str
        :keyword upn:
        :paramtype upn: str
        """
        super(User, self).__init__(**kwargs)
        self.user_object_id = kwargs.get('user_object_id', None)
        self.user_pu_id = kwargs.get('user_pu_id', None)
        self.user_idp = kwargs.get('user_idp', None)
        self.user_alt_sec_id = kwargs.get('user_alt_sec_id', None)
        self.user_iss = kwargs.get('user_iss', None)
        self.user_tenant_id = kwargs.get('user_tenant_id', None)
        self.user_name = kwargs.get('user_name', None)
        self.upn = kwargs.get('upn', None)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

import logging
from abc import ABC, abstractmethod
from pathlib import Path
from typing import TYPE_CHECKING, Any, Dict, Optional

from azure.ai.ml._restclient.v2022_02_01_preview.models import CommandJob as RestCommandJob
from azure.ai.ml._restclient.v2022_02_01_preview.models import JobBaseData
from azure.ai.ml._schema.job.import_job import ImportJobSchema
from azure.ai.ml._utils.utils import is_private_preview_enabled
from azure.ai.ml.constants import JobType
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY, TYPE
from azure.ai.ml.entities._inputs_outputs import Output
from azure.ai.ml.entities._job._input_output_helpers import (
    from_rest_data_outputs,
    from_rest_inputs_to_dataset_literal,
    to_rest_data_outputs,
    to_rest_dataset_literal_inputs,
)
from azure.ai.ml.entities._job.job_io_mixin import JobIOMixin
from azure.ai.ml.entities._util import load_from_dict
from azure.ai.ml.exceptions import MlException

from .job import Job

# avoid circular import error
if TYPE_CHECKING:
    from azure.ai.ml.entities._builders import Import
    from azure.ai.ml.entities._component.import_component import ImportComponent

module_logger = logging.getLogger(__name__)


class ImportSource(ABC):
    def __init__(
        self,
        *,
        type: Optional[str] = None,  # pylint: disable=redefined-builtin
        connection: Optional[str] = None,
    ):
        self.type = type
        self.connection = connection

    @abstractmethod
    def _to_job_inputs(self) -> Dict[str, Optional[str]]:
        pass

    @classmethod
    def _from_job_inputs(cls, job_inputs: Dict[str, str]) -> "ImportSource":
        """Translate job inputs to import source.

        :param job_inputs: The job inputs
        :type job_inputs: Dict[str, str]
        :return: The import source
        :rtype: ImportSource
        """
        type = job_inputs.get("type")  # pylint: disable=redefined-builtin
        connection = job_inputs.get("connection")
        query = job_inputs.get("query")
        path = job_inputs.get("path")

        import_source = (
            DatabaseImportSource(type=type, connection=connection, query=query)
            if query is not None
            else FileImportSource(type=type, connection=connection, path=path)
        )
        return import_source


class DatabaseImportSource(ImportSource):
    def __init__(
        self,
        *,
        type: Optional[str] = None,  # pylint: disable=redefined-builtin
        connection: Optional[str] = None,
        query: Optional[str] = None,
    ):
        ImportSource.__init__(
            self,
            type=type,
            connection=connection,
        )
        self.query = query

    def _to_job_inputs(self) -> Dict[str, Optional[str]]:
        """Translate source to command Inputs.

        :return: The job inputs dict
        :rtype: Dict[str, str]
        """
        inputs = {
            "type": self.type,
            "connection": self.connection,
            "query": self.query,
        }
        return inputs


class FileImportSource(ImportSource):
    def __init__(
        self,
        *,
        type: Optional[str] = None,  # pylint: disable=redefined-builtin
        connection: Optional[str] = None,
        path: Optional[str] = None,
    ):
        ImportSource.__init__(
            self,
            type=type,
            connection=connection,
        )
        self.path = path

    def _to_job_inputs(self) -> Dict[str, Optional[str]]:
        """Translate source to command Inputs.

        :return: The job inputs dict
        :rtype: Dict[str, str]
        """
        inputs = {
            "type": self.type,
            "connection": self.connection,
            "path": self.path,
        }
        return inputs


class ImportJob(Job, JobIOMixin):
    """Import job.

    :param name: Name of the job.
    :type name: str
    :param description: Description of the job.
    :type description: str
    :param display_name: Display name of the job.
    :type display_name: str
    :param experiment_name:  Name of the experiment the job will be created under.
        If None is provided, default will be set to current directory name.
    :type experiment_name: str
    :param source: Input source parameters to the import job.
    :type source: azure.ai.ml.entities.DatabaseImportSource or FileImportSource
    :param output: output data binding used in the job.
    :type output: azure.ai.ml.Output
    :param kwargs: A dictionary of additional configuration parameters.
    :type kwargs: dict
    """

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        description: Optional[str] = None,
        display_name: Optional[str] = None,
        experiment_name: Optional[str] = None,
        source: Optional[ImportSource] = None,
        output: Optional[Output] = None,
        **kwargs: Any,
    ):
        kwargs[TYPE] = JobType.IMPORT

        Job.__init__(
            self,
            name=name,
            display_name=display_name,
            description=description,
            experiment_name=experiment_name,
            **kwargs,
        )

        self.source = source
        self.output = output

    def _to_dict(self) -> Dict:
        res: dict = ImportJobSchema(context={BASE_PATH_CONTEXT_KEY: "./"}).dump(self)
        return res

    def _to_rest_object(self) -> JobBaseData:
        # TODO: Remove in PuP
        if not is_private_preview_enabled():
            msg = JobType.IMPORT + " job not supported."
            raise MlException(message=msg, no_personal_data_message=msg)

        _inputs = self.source._to_job_inputs() if self.source is not None else None  # pylint: disable=protected-access
        if self.compute is None:
            msg = "compute cannot be None."
            raise MlException(message=msg, no_personal_data_message=msg)

        properties = RestCommandJob(
            display_name=self.display_name,
            description=self.description,
            compute_id=self.compute,
            experiment_name=self.experiment_name,
            inputs=to_rest_dataset_literal_inputs(_inputs, job_type=self.type),
            outputs=to_rest_data_outputs({"output": self.output}),
            # TODO: Remove in PuP with native import job/component type support in MFE/Designer
            # No longer applicable once new import job type is ready on MFE in PuP
            # command and environment are required as we use command type for import
            # command can be random string and the particular environment name here is defined as default in MFE
            #   public const string DefaultEnvironmentName = "AzureML-sklearn-0.24-ubuntu18.04-py37-cpu";
            # which is considered valid environment in MFE unless MFE changes current default logic
            # but chance should be very low in PrP
            command="import",
            environment_id=self.compute.replace(
                "/computes/DataFactory", "/environments/AzureML-sklearn-0.24-ubuntu18.04-py37-cpu"
            ),
        )
        result = JobBaseData(properties=properties)
        result.name = self.name
        return result

    @classmethod
    def _load_from_dict(cls, data: Dict, context: Dict, additional_message: str, **kwargs: Any) -> "ImportJob":
        loaded_data = load_from_dict(ImportJobSchema, data, context, additional_message, **kwargs)
        return ImportJob(base_path=context[BASE_PATH_CONTEXT_KEY], **loaded_data)

    @classmethod
    def _load_from_rest(cls, obj: JobBaseData) -> "ImportJob":
        rest_command_job: RestCommandJob = obj.properties
        outputs = from_rest_data_outputs(rest_command_job.outputs)
        inputs = from_rest_inputs_to_dataset_literal(rest_command_job.inputs)

        import_job = ImportJob(
            name=obj.name,
            id=obj.id,
            display_name=rest_command_job.display_name,
            description=rest_command_job.description,
            experiment_name=rest_command_job.experiment_name,
            status=rest_command_job.status,
            creation_context=obj.system_data,
            source=ImportSource._from_job_inputs(inputs),  # pylint: disable=protected-access
            output=outputs["output"] if "output" in outputs else None,
        )
        return import_job

    def _to_component(self, context: Optional[Dict] = None, **kwargs: Any) -> "ImportComponent":
        """Translate a import job to component.

        :param context: Context of import job YAML file.
        :type context: dict
        :return: Translated import component.
        :rtype: ImportComponent
        """
        from azure.ai.ml.entities._component.import_component import ImportComponent

        pipeline_job_dict = kwargs.get("pipeline_job_dict", {})
        context = context or {BASE_PATH_CONTEXT_KEY: Path("import/")}

        _inputs = self.source._to_job_inputs() if self.source is not None else None  # pylint: disable=protected-access

        # Create anonymous command component with default version as 1
        return ImportComponent(
            is_anonymous=True,
            base_path=context[BASE_PATH_CONTEXT_KEY],
            description=self.description,
            source=self._to_inputs(
                inputs=_inputs,
                pipeline_job_dict=pipeline_job_dict,
            ),
            output=self._to_outputs(outputs={"output": self.output}, pipeline_job_dict=pipeline_job_dict)["output"],
        )

    def _to_node(self, context: Optional[Dict] = None, **kwargs: Any) -> "Import":
        """Translate a import job to a pipeline node.

        :param context: Context of import job YAML file.
        :type context: dict
        :return: Translated import node.
        :rtype: Import
        """
        from azure.ai.ml.entities._builders import Import

        component = self._to_component(context, **kwargs)
        _inputs = self.source._to_job_inputs() if self.source is not None else None  # pylint: disable=protected-access
        return Import(
            component=component,
            compute=self.compute,
            inputs=_inputs,
            outputs={"output": self.output},
            description=self.description,
            display_name=self.display_name,
            properties=self.properties,
        )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from pathlib import Path
from typing import Any, Dict, Optional, Union

from marshmallow import Schema

from azure.ai.ml._schema.component.import_component import ImportComponentSchema
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY, COMPONENT_TYPE
from azure.ai.ml.constants._component import NodeType

from ..._schema import PathAwareSchema
from ..._utils.utils import parse_args_description_from_docstring
from .._util import convert_ordered_dict_to_dict
from .component import Component


class ImportComponent(Component):
    """Import component version, used to define an import component.

    :param name: Name of the component.
    :type name: str
    :param version: Version of the component.
    :type version: str
    :param description: Description of the component.
    :type description: str
    :param tags: Tag dictionary. Tags can be added, removed, and updated.
    :type tags: dict
    :param display_name: Display name of the component.
    :type display_name: str
    :param source: Input source parameters of the component.
    :type source: dict
    :param output: Output of the component.
    :type output: dict
    :param is_deterministic: Whether the command component is deterministic. Defaults to True.
    :type is_deterministic: bool
    :param kwargs: Additional parameters for the import component.
    """

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        version: Optional[str] = None,
        description: Optional[str] = None,
        tags: Optional[Dict] = None,
        display_name: Optional[str] = None,
        source: Optional[Dict] = None,
        output: Optional[Dict] = None,
        is_deterministic: bool = True,
        **kwargs: Any,
    ) -> None:
        kwargs[COMPONENT_TYPE] = NodeType.IMPORT
        # Set default base path
        if BASE_PATH_CONTEXT_KEY not in kwargs:
            kwargs[BASE_PATH_CONTEXT_KEY] = Path(".")

        super().__init__(
            name=name,
            version=version,
            description=description,
            tags=tags,
            display_name=display_name,
            inputs=source,
            outputs={"output": output} if output else None,
            is_deterministic=is_deterministic,
            **kwargs,
        )

        self.source = source
        self.output = output

    def _to_dict(self) -> Dict:
        # TODO: Bug Item number: 2897665
        res: Dict = convert_ordered_dict_to_dict(  # type: ignore
            {**self._other_parameter, **super(ImportComponent, self)._to_dict()}
        )
        return res

    @classmethod
    def _create_schema_for_validation(cls, context: Any) -> Union[PathAwareSchema, Schema]:
        return ImportComponentSchema(context=context)

    @classmethod
    def _parse_args_description_from_docstring(cls, docstring: str) -> Dict:
        res: dict = parse_args_description_from_docstring(docstring)
        return res

    def __str__(self) -> str:
        try:
            toYaml: str = self._to_yaml()
            return toYaml
        except BaseException:  # pylint: disable=W0718
            toStr: str = super(ImportComponent, self).__str__()
            return toStr



import argparse
import os
import sys

import pyspark
from pyspark.ml.classification import *
from pyspark.ml.evaluation import *
from pyspark.ml.feature import *
from pyspark.sql.functions import *
from pyspark.sql.types import DoubleType, IntegerType, StringType, StructField, StructType

# Get args
parser = argparse.ArgumentParser()
parser.add_argument("--input_path")
parser.add_argument("--regularization_rate")
parser.add_argument("--output_path")
args, _ = parser.parse_known_args()
input_path = args.input_path
regularization_rate = args.regularization_rate
output_path = args.output_path

print("input_path: {}".format(input_path))
print("regularization_rate: {}".format(regularization_rate))
print("output_path: {}".format(output_path))

# start Spark session
spark = pyspark.sql.SparkSession.builder.appName("Iris").getOrCreate()

# print runtime versions
print("****************")
print("Python version: {}".format(sys.version))
print("Spark version: {}".format(spark.version))
print("****************")

# load iris.csv into Spark dataframe
schema = StructType(
    [
        StructField("sepal-length", DoubleType()),
        StructField("sepal-width", DoubleType()),
        StructField("petal-length", DoubleType()),
        StructField("petal-width", DoubleType()),
        StructField("class", StringType()),
    ]
)

data = spark.read.csv(input_path, header=True, schema=schema)

print("First 10 rows of Iris dataset:")
data.show(10)

# vectorize all numerical columns into a single feature column
feature_cols = data.columns[:-1]
assembler = pyspark.ml.feature.VectorAssembler(inputCols=feature_cols, outputCol="features")
data = assembler.transform(data)

# convert text labels into indices
data = data.select(["features", "class"])
label_indexer = pyspark.ml.feature.StringIndexer(inputCol="class", outputCol="label").fit(data)
data = label_indexer.transform(data)

# only select the features and label column
data = data.select(["features", "label"])
print("Reading for machine learning")
data.show(10)

# use Logistic Regression to train on the training set
train, test = data.randomSplit([0.70, 0.30])
reg = float(regularization_rate)
lr = pyspark.ml.classification.LogisticRegression(regParam=reg)
model = lr.fit(train)

model.save(os.path.join(output_path, "iris.model"))

# predict on the test set
prediction = model.transform(test)
print("Prediction")
prediction.show(10)

# evaluate the accuracy of the model using the test set
evaluator = pyspark.ml.evaluation.MulticlassClassificationEvaluator(metricName="accuracy")
accuracy = evaluator.evaluate(prediction)

print()
print("#####################################")
print("Regularization rate is {}".format(reg))
print("Accuracy is {}".format(accuracy))
print("#####################################")
print()



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from abc import ABC
from typing import Any, Dict, List, Optional, Union

from azure.ai.ml._restclient.v2023_04_01_preview.models import (
    LogVerbosity,
    NlpLearningRateScheduler,
    SamplingAlgorithmType,
)
from azure.ai.ml._utils.utils import camel_to_snake
from azure.ai.ml.entities._inputs_outputs import Input
from azure.ai.ml.entities._job.automl.automl_vertical import AutoMLVertical
from azure.ai.ml.entities._job.automl.nlp.nlp_featurization_settings import NlpFeaturizationSettings
from azure.ai.ml.entities._job.automl.nlp.nlp_fixed_parameters import NlpFixedParameters
from azure.ai.ml.entities._job.automl.nlp.nlp_limit_settings import NlpLimitSettings
from azure.ai.ml.entities._job.automl.nlp.nlp_search_space import NlpSearchSpace
from azure.ai.ml.entities._job.automl.nlp.nlp_sweep_settings import NlpSweepSettings
from azure.ai.ml.entities._job.automl.search_space import SearchSpace
from azure.ai.ml.entities._job.automl.utils import cast_to_specific_search_space
from azure.ai.ml.entities._job.sweep.early_termination_policy import EarlyTerminationPolicy
from azure.ai.ml.exceptions import ErrorCategory, ErrorTarget, ValidationException


# pylint: disable=too-many-instance-attributes,protected-access
class AutoMLNLPJob(AutoMLVertical, ABC):
    """Base class for AutoML NLP jobs.

    You should not instantiate this class directly. Instead you should
    create classes for specific NLP Jobs.

    :param task_type: NLP task type, must be one of 'TextClassification',
        'TextClassificationMultilabel', or 'TextNER'
    :type task_type: str
    :param primary_metric: Primary metric to display from NLP job
    :type primary_metric: str
    :param training_data: Training data
    :type training_data: Input
    :param validation_data: Validation data
    :type validation_data: Input
    :param target_column_name: Column name of the target column, defaults to None
    :type target_column_name: Optional[str]
    :param log_verbosity: The degree of verbosity used in logging, defaults to None,
        must be one of 'NotSet', 'Debug', 'Info', 'Warning', 'Error', 'Critical', or None
    :type log_verbosity: Optional[str]
    :param featurization: Featurization settings used for NLP job, defaults to None
    :type featurization: Optional[~azure.ai.ml.automl.NlpFeaturizationSettings]
    :param limits: Limit settings for NLP jobs, defaults to None
    :type limits: Optional[~azure.ai.ml.automl.NlpLimitSettings]
    :param sweep: Sweep settings used for NLP job, defaults to None
    :type sweep: Optional[~azure.ai.ml.automl.NlpSweepSettings]
    :param training_parameters: Fixed parameters for the training of all candidates.
        , defaults to None
    :type training_parameters: Optional[~azure.ai.ml.automl.NlpFixedParameters]
    :param search_space: Search space(s) to sweep over for NLP sweep jobs, defaults to None
    :type search_space: Optional[List[~azure.ai.ml.automl.NlpSearchSpace]]
    """

    def __init__(
        self,
        *,
        task_type: str,
        primary_metric: str,
        training_data: Optional[Input],
        validation_data: Optional[Input],
        target_column_name: Optional[str] = None,
        log_verbosity: Optional[str] = None,
        featurization: Optional[NlpFeaturizationSettings] = None,
        limits: Optional[NlpLimitSettings] = None,
        sweep: Optional[NlpSweepSettings] = None,
        training_parameters: Optional[NlpFixedParameters] = None,
        search_space: Optional[List[NlpSearchSpace]] = None,
        **kwargs: Any,
    ):
        self._training_parameters: Optional[NlpFixedParameters] = None

        super().__init__(
            task_type, training_data=training_data, validation_data=validation_data, **kwargs  # type: ignore
        )
        self.log_verbosity = log_verbosity
        self._primary_metric: str = ""
        self.primary_metric = primary_metric

        self.target_column_name = target_column_name

        self._featurization = featurization
        self._limits = limits or NlpLimitSettings()
        self._sweep = sweep
        self.training_parameters = training_parameters  # via setter method.
        self._search_space = search_space

    @property
    def training_parameters(self) -> Optional[NlpFixedParameters]:
        """Parameters that are used for all submitted jobs.

        :return: fixed training parameters for NLP jobs
        :rtype: ~azure.ai.ml.automl.NlpFixedParameters
        """
        return self._training_parameters

    @training_parameters.setter
    def training_parameters(self, value: Union[Dict, NlpFixedParameters]) -> None:
        if value is None:
            self._training_parameters = None
        elif isinstance(value, NlpFixedParameters):
            self._training_parameters = value
            # Convert parameters from snake case to enum.
            self.set_training_parameters(learning_rate_scheduler=value.learning_rate_scheduler)
        else:
            if not isinstance(value, dict):
                msg = "Expected a dictionary for nlp training parameters."
                raise ValidationException(
                    message=msg,
                    no_personal_data_message=msg,
                    target=ErrorTarget.AUTOML,
                    error_category=ErrorCategory.USER_ERROR,
                )
            self.set_training_parameters(**value)

    @property
    def search_space(self) -> Optional[List[NlpSearchSpace]]:
        """Search space(s) to sweep over for NLP sweep jobs

        :return: list of search spaces to sweep over for NLP jobs
        :rtype: List[~azure.ai.ml.automl.NlpSearchSpace]
        """
        return self._search_space

    @search_space.setter
    def search_space(self, value: Union[List[dict], List[SearchSpace]]) -> None:
        if not isinstance(value, list):
            msg = "Expected a list for search space."
            raise ValidationException(
                message=msg,
                no_personal_data_message=msg,
                target=ErrorTarget.AUTOML,
                error_category=ErrorCategory.USER_ERROR,
            )

        all_dict_type = all(isinstance(item, dict) for item in value)
        all_search_space_type = all(isinstance(item, SearchSpace) for item in value)

        if not (all_search_space_type or all_dict_type):
            msg = "Expected all items in the list to be either dictionaries or SearchSpace objects."
            raise ValidationException(
                message=msg,
                no_personal_data_message=msg,
                target=ErrorTarget.AUTOML,
                error_category=ErrorCategory.USER_ERROR,
            )

        self._search_space = [
            cast_to_specific_search_space(item, NlpSearchSpace, self.task_type) for item in value  # type: ignore
        ]

    @property
    def primary_metric(self) -> str:
        """Primary metric to display from NLP job

        :return: primary metric to display
        :rtype: str
        """
        return self._primary_metric

    @primary_metric.setter
    def primary_metric(self, value: str) -> None:
        self._primary_metric = value

    @property
    def log_verbosity(self) -> LogVerbosity:
        """Log verbosity configuration

        :return: the degree of verbosity used in logging
        :rtype: ~azure.mgmt.machinelearningservices.models.LogVerbosity
        """
        return self._log_verbosity

    @log_verbosity.setter
    def log_verbosity(self, value: Union[str, LogVerbosity]) -> None:
        self._log_verbosity = None if value is None else LogVerbosity[camel_to_snake(value).upper()]

    @property
    def limits(self) -> NlpLimitSettings:
        """Limit settings for NLP jobs

        :return: limit configuration for NLP job
        :rtype: ~azure.ai.ml.automl.NlpLimitSettings
        """
        return self._limits

    @limits.setter
    def limits(self, value: Union[Dict, NlpLimitSettings]) -> None:
        if isinstance(value, NlpLimitSettings):
            self._limits = value
        else:
            if not isinstance(value, dict):
                msg = "Expected a dictionary for limit settings."
                raise ValidationException(
                    message=msg,
                    no_personal_data_message=msg,
                    target=ErrorTarget.AUTOML,
                    error_category=ErrorCategory.USER_ERROR,
                )
            self.set_limits(**value)

    @property
    def sweep(self) -> Optional[NlpSweepSettings]:
        """Sweep settings used for NLP job

        :return: sweep settings
        :rtype: ~azure.ai.ml.automl.NlpSweepSettings
        """
        return self._sweep

    @sweep.setter
    def sweep(self, value: Union[Dict, NlpSweepSettings]) -> None:
        if isinstance(value, NlpSweepSettings):
            self._sweep = value
        else:
            if not isinstance(value, dict):
                msg = "Expected a dictionary for sweep settings."
                raise ValidationException(
                    message=msg,
                    no_personal_data_message=msg,
                    target=ErrorTarget.AUTOML,
                    error_category=ErrorCategory.USER_ERROR,
                )
            self.set_sweep(**value)

    @property
    def featurization(self) -> Optional[NlpFeaturizationSettings]:
        """Featurization settings used for NLP job

        :return: featurization settings
        :rtype: ~azure.ai.ml.automl.NlpFeaturizationSettings
        """
        return self._featurization

    @featurization.setter
    def featurization(self, value: Union[Dict, NlpFeaturizationSettings]) -> None:
        if isinstance(value, NlpFeaturizationSettings):
            self._featurization = value
        else:
            if not isinstance(value, dict):
                msg = "Expected a dictionary for featurization settings."
                raise ValidationException(
                    message=msg,
                    no_personal_data_message=msg,
                    target=ErrorTarget.AUTOML,
                    error_category=ErrorCategory.USER_ERROR,
                )
            self.set_featurization(**value)

    def set_data(self, *, training_data: Input, target_column_name: str, validation_data: Input) -> None:
        """Define data configuration for NLP job

        :keyword training_data: Training data
        :type training_data: ~azure.ai.ml.Input
        :keyword target_column_name: Column name of the target column.
        :type target_column_name: str
        :keyword validation_data: Validation data
        :type validation_data: ~azure.ai.ml.Input
        """
        # Properties for NlpVerticalDataSettings
        self.target_column_name = target_column_name
        self.training_data = training_data
        self.validation_data = validation_data

    def set_limits(
        self,
        *,
        max_trials: int = 1,
        max_concurrent_trials: int = 1,
        max_nodes: int = 1,
        timeout_minutes: Optional[int] = None,
        trial_timeout_minutes: Optional[int] = None,
    ) -> None:
        """Define limit configuration for AutoML NLP job

        :keyword max_trials: Maximum number of AutoML iterations, defaults to 1
        :type max_trials: int, optional
        :keyword max_concurrent_trials: Maximum number of concurrent AutoML iterations, defaults to 1
        :type max_concurrent_trials: int, optional
        :keyword max_nodes: Maximum number of nodes used for sweep, defaults to 1
        :type max_nodes: int, optional
        :keyword timeout_minutes: Timeout for the AutoML job, defaults to None
        :type timeout_minutes: Optional[int]
        :keyword trial_timeout_minutes: Timeout for each AutoML trial, defaults to None
        :type trial_timeout_minutes: Optional[int]
        """
        self._limits = NlpLimitSettings(
            max_trials=max_trials,
            max_concurrent_trials=max_concurrent_trials,
            max_nodes=max_nodes,
            timeout_minutes=timeout_minutes,
            trial_timeout_minutes=trial_timeout_minutes,
        )

    def set_sweep(
        self,
        *,
        sampling_algorithm: Union[str, SamplingAlgorithmType],
        early_termination: Optional[EarlyTerminationPolicy] = None,
    ) -> None:
        """Define sweep configuration for AutoML NLP job

        :keyword sampling_algorithm: Required. Specifies type of hyperparameter sampling algorithm.
            Possible values include: "Grid", "Random", and "Bayesian".
        :type sampling_algorithm: Union[str, ~azure.ai.ml.automl.SamplingAlgorithmType]
        :keyword early_termination: Optional. early termination policy to end poorly performing training candidates,
            defaults to None.
        :type early_termination: Optional[~azure.mgmt.machinelearningservices.models.EarlyTerminationPolicy]
        """
        if self._sweep:
            self._sweep.sampling_algorithm = sampling_algorithm
        else:
            self._sweep = NlpSweepSettings(sampling_algorithm=sampling_algorithm)

        self._sweep.early_termination = early_termination or self._sweep.early_termination

    def set_training_parameters(
        self,
        *,
        gradient_accumulation_steps: Optional[int] = None,
        learning_rate: Optional[float] = None,
        learning_rate_scheduler: Optional[Union[str, NlpLearningRateScheduler]] = None,
        model_name: Optional[str] = None,
        number_of_epochs: Optional[int] = None,
        training_batch_size: Optional[int] = None,
        validation_batch_size: Optional[int] = None,
        warmup_ratio: Optional[float] = None,
        weight_decay: Optional[float] = None,
    ) -> None:
        """Fix certain training parameters throughout the training procedure for all candidates.

        :keyword gradient_accumulation_steps: number of steps over which to accumulate gradients before a backward
            pass. This must be a positive integer., defaults to None
        :type gradient_accumulation_steps: Optional[int]
        :keyword learning_rate: initial learning rate. Must be a float in (0, 1)., defaults to None
        :type learning_rate: Optional[float]
        :keyword learning_rate_scheduler: the type of learning rate scheduler. Must choose from 'linear', 'cosine',
            'cosine_with_restarts', 'polynomial', 'constant', and 'constant_with_warmup'., defaults to None
        :type learning_rate_scheduler: Optional[Union[str, ~azure.ai.ml.automl.NlpLearningRateScheduler]]
        :keyword model_name: the model name to use during training. Must choose from 'bert-base-cased',
            'bert-base-uncased', 'bert-base-multilingual-cased', 'bert-base-german-cased', 'bert-large-cased',
            'bert-large-uncased', 'distilbert-base-cased', 'distilbert-base-uncased', 'roberta-base', 'roberta-large',
            'distilroberta-base', 'xlm-roberta-base', 'xlm-roberta-large', xlnet-base-cased', and 'xlnet-large-cased'.,
            defaults to None
        :type model_name: Optional[str]
        :keyword number_of_epochs: the number of epochs to train with. Must be a positive integer., defaults to None
        :type number_of_epochs: Optional[int]
        :keyword training_batch_size: the batch size during training. Must be a positive integer., defaults to None
        :type training_batch_size: Optional[int]
        :keyword validation_batch_size: the batch size during validation. Must be a positive integer., defaults to None
        :type validation_batch_size: Optional[int]
        :keyword warmup_ratio: ratio of total training steps used for a linear warmup from 0 to learning_rate.
            Must be a float in [0, 1]., defaults to None
        :type warmup_ratio: Optional[float]
        :keyword weight_decay: value of weight decay when optimizer is sgd, adam, or adamw. This must be a float in
            the range [0, 1]., defaults to None
        :type weight_decay: Optional[float]
        """
        self._training_parameters = self._training_parameters or NlpFixedParameters()

        self._training_parameters.gradient_accumulation_steps = (
            gradient_accumulation_steps
            if gradient_accumulation_steps is not None
            else self._training_parameters.gradient_accumulation_steps
        )

        self._training_parameters.learning_rate = (
            learning_rate if learning_rate is not None else self._training_parameters.learning_rate
        )

        self._training_parameters.learning_rate_scheduler = (
            NlpLearningRateScheduler[camel_to_snake(learning_rate_scheduler).upper()]
            if learning_rate_scheduler is not None
            else self._training_parameters.learning_rate_scheduler
        )

        self._training_parameters.model_name = (
            model_name if model_name is not None else self._training_parameters.model_name
        )

        self._training_parameters.number_of_epochs = (
            number_of_epochs if number_of_epochs is not None else self._training_parameters.number_of_epochs
        )

        self._training_parameters.training_batch_size = (
            training_batch_size if training_batch_size is not None else self._training_parameters.training_batch_size
        )

        self._training_parameters.validation_batch_size = (
            validation_batch_size
            if validation_batch_size is not None
            else self._training_parameters.validation_batch_size
        )

        self._training_parameters.warmup_ratio = (
            warmup_ratio if warmup_ratio is not None else self._training_parameters.warmup_ratio
        )

        self._training_parameters.weight_decay = (
            weight_decay if weight_decay is not None else self._training_parameters.weight_decay
        )

    def set_featurization(self, *, dataset_language: Optional[str] = None) -> None:
        """Define featurization configuration for AutoML NLP job.

        :keyword dataset_language: Language of the dataset, defaults to None
        :type dataset_language: Optional[str]
        """
        self._featurization = NlpFeaturizationSettings(
            dataset_language=dataset_language,
        )

    def extend_search_space(self, value: Union[SearchSpace, List[SearchSpace]]) -> None:
        """Add (a) search space(s) for an AutoML NLP job.

        :param value: either a SearchSpace object or a list of SearchSpace objects with nlp-specific parameters.
        :type value: Union[~azure.ai.ml.automl.SearchSpace, List[~azure.ai.ml.automl.SearchSpace]]
        """
        self._search_space = self._search_space or []
        if isinstance(value, list):
            self._search_space.extend(
                [cast_to_specific_search_space(item, NlpSearchSpace, self.task_type) for item in value]  # type: ignore
            )
        else:
            self._search_space.append(
                cast_to_specific_search_space(value, NlpSearchSpace, self.task_type)  # type: ignore
            )

    @classmethod
    def _get_search_space_from_str(cls, search_space_str: Optional[str]) -> Optional[List]:
        if search_space_str is not None:
            return [NlpSearchSpace._from_rest_object(entry) for entry in search_space_str if entry is not None]
        return None

    def _restore_data_inputs(self) -> None:
        """Restore MLTableJobInputs to Inputs within data_settings.

        self.training_data and self.validation_data should reflect what user passed in (Input) Once we get response back
        from service (as MLTableJobInput), we should set responsible ones back to Input
        """
        super()._restore_data_inputs()
        self.training_data = self.training_data if self.training_data else None  # type: ignore
        self.validation_data = self.validation_data if self.validation_data else None  # type: ignore

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, AutoMLNLPJob):
            return NotImplemented

        return (
            self.primary_metric == other.primary_metric
            and self.log_verbosity == other.log_verbosity
            and self.training_data == other.training_data
            and self.validation_data == other.validation_data
            and self._featurization == other._featurization
            and self._limits == other._limits
            and self._sweep == other._sweep
            and self._training_parameters == other._training_parameters
            and self._search_space == other._search_space
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)



from pathlib import Path

import pytest
import yaml

from azure.ai.ml._schema._deployment.batch.batch_deployment import BatchDeploymentSchema
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY
from azure.ai.ml.constants._deployment import BatchDeploymentOutputAction
from azure.ai.ml.entities import BatchDeployment
from azure.ai.ml.entities._util import load_from_dict


def load_batch_deployment_entity_from_yaml(path: str, context={}) -> BatchDeployment:
    """batch deployment yaml -> batch deployment entity"""
    with open(path, "r") as f:
        cfg = yaml.safe_load(f)
    context.update({BASE_PATH_CONTEXT_KEY: Path(path).parent})
    deployment = load_from_dict(BatchDeploymentSchema, cfg, context)
    return deployment


@pytest.mark.unittest
@pytest.mark.production_experiences_test
class TestBatchDeploymentSchema:
    def test_serialize_batch_deployment(self) -> None:
        test_path = "./tests/test_configs/deployments/batch/batch_deployment_1.yaml"
        batch_deployment_entity = load_batch_deployment_entity_from_yaml(test_path)

        assert batch_deployment_entity
        assert batch_deployment_entity.environment == "AzureML-sklearn-1.0-ubuntu20.04-py38-cpu:33"
        assert batch_deployment_entity.compute == "cpu-cluster"
        assert batch_deployment_entity.output_action == BatchDeploymentOutputAction.APPEND_ROW
        assert batch_deployment_entity.output_file_name == "append_row.txt"
        assert batch_deployment_entity.error_threshold == 10
        assert batch_deployment_entity.mini_batch_size == 5
        assert batch_deployment_entity.max_concurrency_per_instance == 5
        assert batch_deployment_entity.resources.instance_count == 2

    def test_registry_assets_batch_deployment(self) -> None:
        test_path = "./tests/test_configs/deployments/batch/batch_deployment_registry.yaml"
        batch_deployment_entity = load_batch_deployment_entity_from_yaml(test_path)
        assert batch_deployment_entity
        assert batch_deployment_entity.model == "azureml://registries/testFeed/models/model_version_e2e/versions/1"
        assert (
            batch_deployment_entity.environment
            == "azureml://registries/testfeed/environments/4c99f460-20cd-4821-8745-202aa7555604/versions/93435847-704b-4280-83f3-f735d8b5eff7"
        )
        assert batch_deployment_entity.compute == "cpu-cluster"
        assert batch_deployment_entity.output_action == BatchDeploymentOutputAction.APPEND_ROW
        assert batch_deployment_entity.output_file_name == "append_row.txt"
        assert batch_deployment_entity.error_threshold == 10
        assert batch_deployment_entity.mini_batch_size == 5
        assert batch_deployment_entity.max_concurrency_per_instance == 5
        assert batch_deployment_entity.resources.instance_count == 2
